#!/usr/bin/perl

use strict;

use GnuplotWrapper;

my $EXPERIMENT = "migration_cost_evaluation";
my @PLACEMENTS = (
	{
		id		=> "none_NS2",
		label		=> "No optimization",
		label_offset	=> -7,
		style		=> "base_case_1",
	},
	{
		id		=> "spi_NS2",
		label		=> "SPi",
		label_offset	=> -1,
		style		=> "contribution_1",
	},
);
my %CONTEXT = (placement => '', nodes => '', request_frequency => '');



my @EVALUATIONS = (
	{
		id		=> "unicasts_VS_nodes",
		variable	=> $VARIABLES{nodes},
		value		=> $VALUES{transmittedUnicastPackets},
		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
		key_position	=> "top left",
		y_axis_range	=> "[0:5.5]",
	},
	{
		id		=> "broadcasts_VS_nodes",
		variable	=> $VARIABLES{nodes},
		value		=> $VALUES{transmittedBroadcastPackets},
		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
		key_position	=> "top left",
	},
# 	{
# 		id		=> "traffic_VS_nodes",
# 		variable	=> $VARIABLES{nodes},
# 		value		=> $VALUES{transmittedBytes},
# 		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
# 		key_position	=> "top left",
# 	},
	{
		id		=> "datarate_VS_nodes",
		variable	=> $VARIABLES{nodes},
		value		=> $VALUES{transmittedDatarate},
		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
		key_position	=> "top left",
		y_axis_range	=> "[0:32.5]",
	},
	{
		id		=> "rreqs_VS_nodes",
		variable	=> $VARIABLES{nodes},
		value		=> $VALUES{DYMO_transmittedRREQs},
		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
		key_position	=> "top left",
	},
# 	{
# 		id		=> "rreps_VS_nodes",
# 		variable	=> $VARIABLES{nodes},
# 		value		=> $VALUES{DYMO_transmittedRREPs},
# 		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
# 		key_position	=> "top left",
# 	},
# 	{
# 		id		=> "rerrs_VS_nodes",
# 		variable	=> $VARIABLES{nodes},
# 		value		=> $VALUES{DYMO_transmittedRERRs},
# 		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
# 		key_position	=> "top left",
# 	},
	{
		id		=> "slp_queries_VS_nodes",
		variable	=> $VARIABLES{nodes},
		value		=> $VALUES{SLP_totalQueries},
		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
		key_position	=> "top left",
	},
	{
		id		=> "service_request_delay_VS_nodes",
		variable	=> $VARIABLES{nodes},
		value		=> $VALUES{SERVICE_averageSuccessfulDataRequestDelay},
		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
		key_position	=> "bottom right",
		y_axis_range	=> "[0.008:1.2]",
	},
	{
		id		=> "service_recall_VS_nodes",
		variable	=> $VARIABLES{nodes},
		value		=> $VALUES{SERVICE_requestRecall},
		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
		key_position	=> "bottom left",
	},
	{
		id		=> "service_datarate_VS_nodes",
		variable	=> $VARIABLES{nodes},
		value		=> $VALUES{SERVICE_transmittedDatarate},
		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
		key_position	=> "top left",
		y_axis_range	=> "[0:1.1]",
	},
# 	{
# 		id		=> "average_service_request_hop_count_VS_nodes",
# 		variable	=> $VARIABLES{nodes},
# 		value		=> $VALUES{SERVICE_averageRequestHopCount},
# 		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
# 		key_position	=> "top left",
# 	},
# 	{
# 		id		=> "service_instances_VS_nodes",
# 		variable	=> $VARIABLES{nodes},
# 		value		=> $VALUES{SERVICE_hostedServices},
# 		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
# 		key_position	=> "top right",
# 	},
# 	{
# 		id		=> "service_replications_VS_nodes",
# 		variable	=> $VARIABLES{nodes},
# 		value		=> $VALUES{SERVICE_totalReplications},
# 		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
# 		key_position	=> "top left",
# 	},
);
my @SERVICE_SIZES = (100000);
my @SYNC_RATIO = ("0.050");
my @NODES = (16, 25, 36, 49, 64, 81, 100);
my @ACTIVE_NODE_RATIOS = ("0.5");
# my @REQUEST_FREQUENCIES = ("0.1", "0.5", "1.0", "2.0", "5.0");
my @REQUEST_FREQUENCIES = ("1.0");

if(scalar(@ARGV) == 0) {
	generate_summary_plots($EXPERIMENT, \@EVALUATIONS, \@PLACEMENTS, \@SERVICE_SIZES, \@SYNC_RATIO, \@NODES, \@ACTIVE_NODE_RATIOS, \@REQUEST_FREQUENCIES, \%CONTEXT);
}



my @EVALUATIONS = (
	{
		id		=> "datarate_VS_time",
		variable	=> $VARIABLES{current_time},
		value		=> $VALUES{transmittedDatarate},
		plot_type	=> $PLOT_TYPE_TIME_SERIES,
		key_position	=> "",
	},
	{
		id		=> "service_request_delay_VS_time",
		variable	=> $VARIABLES{current_time},
		value		=> $VALUES{SERVICE_averageSuccessfulDataRequestDelay},
		plot_type	=> $PLOT_TYPE_TIME_SERIES,
		key_position	=> "",
	},
	{
		id		=> "service_recall_VS_time",
		variable	=> $VARIABLES{current_time},
		value		=> $VALUES{SERVICE_requestRecall},
		plot_type	=> $PLOT_TYPE_TIME_SERIES,
		key_position	=> "",
	},
	{
		id		=> "service_datarate_VS_time",
		variable	=> $VARIABLES{current_time},
		value		=> $VALUES{SERVICE_transmittedDatarate},
		plot_type	=> $PLOT_TYPE_TIME_SERIES,
		key_position	=> "",
	},
	{
		id		=> "service_instances_VS_time",
		variable	=> $VARIABLES{current_time},
		value		=> $VALUES{SERVICE_hostedServices},
		plot_type	=> $PLOT_TYPE_TIME_SERIES,
		key_position	=> "",
	},
	{
		id		=> "service_replications_VS_time",
		variable	=> $VARIABLES{current_time},
		value		=> $VALUES{SERVICE_totalReplications},
		plot_type	=> $PLOT_TYPE_TIME_SERIES,
		key_position	=> "",
	},
	{
		id		=> "service_migrations_VS_time",
		variable	=> $VARIABLES{current_time},
		value		=> $VALUES{SERVICE_totalMigrations},
		plot_type	=> $PLOT_TYPE_TIME_SERIES,
		key_position	=> "",
	},
	{
		id		=> "service_cost_VS_time",
		variable	=> $VARIABLES{current_time},
		value		=> $VALUES{PLACEMENT_currentServiceCost},
		plot_type	=> $PLOT_TYPE_TIME_SERIES,
		key_position	=> "",
	},
	{
		id		=> "optimal_service_cost_VS_time",
		variable	=> $VARIABLES{current_time},
		value		=> $VALUES{PLACEMENT_currentlyOptimalServiceCost},
		plot_type	=> $PLOT_TYPE_TIME_SERIES,
		key_position	=> "",
	},
	{
		id		=> "optimal_service_instances_VS_time",
		variable	=> $VARIABLES{current_time},
		value		=> $VALUES{PLACEMENT_currentlyOptimalNumberOfHosts},
		plot_type	=> $PLOT_TYPE_TIME_SERIES,
		key_position	=> "",
	},
	{
		id		=> "placement_adaptation_cost_VS_time",
		variable	=> $VARIABLES{current_time},
		value		=> $VALUES{PLACEMENT_currentlyEstimatedAdaptationCost},
		plot_type	=> $PLOT_TYPE_TIME_SERIES,
		key_position	=> "",
	},
	{
		id		=> "placement_adaptations_VS_time",
		variable	=> $VARIABLES{current_time},
		value		=> $VALUES{PLACEMENT_totalAdaptations},
		plot_type	=> $PLOT_TYPE_TIME_SERIES,
		key_position	=> "",
	},
);

if(scalar(@ARGV) > 0) {
	generate_time_plots($EXPERIMENT, \@EVALUATIONS, $ARGV[0]);
}
