#!/usr/bin/perl

my $eps_file = "placement_over_time.eps";
my $pdf_file = "placement_over_time.pdf";

print "Generating $pdf_file...\n";
open(GNUPLOT, "| gnuplot") or die "can't open gnuplot: $!";

print GNUPLOT <<EOF or die "can't print to gnuplot: $!";
#
# Output control
#

set terminal postscript eps enhanced color 24
set output "$eps_file"



#
# Input control
#

set datafile separator ","



#
# Scale / aspect ratio
#

set size ratio 0.5 1.0,1.6180339887	# Golden ratio



#
# Axis: labels, ranges, scales
#

set xlabel "Time (s)"
set xrange [0:1200]
#set logscale x

#
# Legend
#

set key off
#set key top right
#set key top left
#set key bottom left
#set key outside
#set key spacing 0.8
#set key width -4
#set key height 1
#set key box



#
# Line and point styles
#

set style line 1 lt 4 lc rgbcolor "blue" lw 2 pt 2
set style line 2 lt 4 lc rgbcolor "sea-green" lw 2 pt 6
set style line 3 lt 4 lc rgbcolor "red" lw 2 pt 2



#
# Plot
#

#current_time(1)
#SERVICE_averageSuccessfulDataRequestDelay(79)
#SERVICE_requestRecall(91)
#SERVICE_averageRequestHopCount(95)
#SERVICE_currentlyHostedServices(118)
#SERVICE_totalReplications(120),SERVICE_successfulReplications(121),SERVICE_replicationSuccessRate(122),SERVICE_totalMigrations(123),SERVICE_successfulMigrations(124),SERVICE_migrationSuccessRate(125),PLACEMENT_currentServiceCost(126),PLACEMENT_currentlyOptimalServiceCost(127),PLACEMENT_currentlyOptimalNumberOfHosts(128),PLACEMENT_currentlyEstimatedAdaptationCost(129),PLACEMENT_totalAdaptations(130),PLACEMENT_successfulAdaptations(131)

set ylabel "# Service Instances" 1
set yrange [0:30]
#set logscale y

plot \\
"< egrep '^[0-9]' placement_over_time.csv" \\
using 1:118 with linespoints ls 1, \\
'' using 1:128 with points ls 2

set ylabel "Service Provisioning Cost (kB/s)" 1
set yrange [0:100]
#set logscale y

plot \\
"< egrep '^[0-9]' placement_over_time.csv" \\
using 1:(\$126/1000) with linespoints ls 1, \\
'' using 1:(\$127/1000) with linespoints ls 2

set ylabel "Est. Adaptation Cost (kB/s)" 1
set yrange [0:500]
#set logscale y

plot \\
"< egrep '^[0-9]' placement_over_time.csv" \\
using 1:(\$129/1000) with linespoints ls 3

set ylabel "# Adaptations" 1
set yrange [0:21]
#set logscale y

plot \\
"< egrep '^[0-9]' placement_over_time.csv" \\
using 1:130 with linespoints ls 1, \\
'' using 1:131 with linespoints ls 2

set ylabel "# Replications" 1
set yrange [0:21]
#set logscale y

plot \\
"< egrep '^[0-9]' placement_over_time.csv" \\
using 1:120 with linespoints ls 1, \\
'' using 1:121 with linespoints ls 2

set ylabel "# Migrations" 1
set yrange [0:21]
#set logscale y

plot \\
"< egrep '^[0-9]' placement_over_time.csv" \\
using 1:123 with linespoints ls 1, \\
'' using 1:124 with linespoints ls 2

set ylabel "Average Distance (hops)" 1
set yrange [0:5]
#set logscale y

plot \\
"< egrep '^[0-9]' placement_over_time.csv" \\
using 1:95 with linespoints ls 1

set ylabel "Average Recall" 1
set yrange [0:1.1]
#set logscale y

plot \\
"< egrep '^[0-9]' placement_over_time.csv" \\
using 1:91 with linespoints ls 1

set ylabel "Average Round-trip Time (s)" 1
set yrange [0:0.2]
#set logscale y

plot \\
"< egrep '^[0-9]' placement_over_time.csv" \\
using 1:79 with linespoints ls 1

set ylabel "Traffic (kB/s)" 1
set yrange [0:600]
#set logscale y

plot \\
"< egrep '^[0-9]' placement_over_time_TRAFFIC.csv" \\
using 1:(\$2/1000) with linespoints ls 1

set ylabel "Service Requests" 1
set yrange [0:120]
#set logscale y

plot \\
"< egrep '^[0-9]' placement_over_time_TRAFFIC.csv" \\
using 1:4 with linespoints ls 1

set ylabel "Service Traffic (kB/s)" 1
set yrange [0:200]
#set logscale y

plot \\
"< egrep '^[0-9]' placement_over_time_TRAFFIC.csv" \\
using 1:(\$6/1000) with linespoints ls 1

EOF

while(<GNUPLOT>) {print;}
close GNUPLOT;
print `ps2pdf $eps_file`;
print `pdfcrop $pdf_file $pdf_file > /dev/null`;
unlink $eps_file;
