#!/usr/bin/perl

use strict;

use GnuplotWrapper;

#require "gnuplot_wrapper.pl";

my $EXPERIMENT = "placement_variable_demand_evaluation";
my @PLACEMENTS = (
	{
		id		=> "none_NS2",
		label		=> "No placement",
		label_offset	=> -5,
		style		=> "base_case_1",
	},
	{
		id		=> "topocenter_NS2",
		label		=> "TopoCenter(1) Migration",
		label_offset	=> -11,
		style		=> "major_alternative_1",
	},
	{
		id		=> "tree-topology_NS2",
		label		=> "Tree-Topology Migration",
		label_offset	=> -11,
		style		=> "major_alternative_2",
	},
	{
		id		=> "asg_simple_NS2",
		label		=> "ASG / simple",
		label_offset	=> -5,
		style		=> "major_alternative_2",
	},
	{
		id		=> "asg_event_flow_tree_NS2",
		label		=> "ASG / EFT",
		label_offset	=> -3,
		style		=> "major_alternative_1",
	},
	{
		id		=> "spi_gcsi_NS2",
		label		=> "SPi / GCSI",
		label_offset	=> -4,
		style		=> "contribution_2",
	},
	{
		id		=> "spi_gcmi_NS2",
		label		=> "SPi / GCMI",
		label_offset	=> -4,
		style		=> "contribution_1",
	},
);
my %CONTEXT = (placement => '', sync_ratio => '', request_frequency => '');		# FIXME: Ugly switch between filename formats.
# my %CONTEXT = (placement => '', active_node_ratio => '', request_frequency => '');	# FIXME: Ugly switch between filename formats.



my @EVALUATIONS = (
# 	{
# 		id		=> "traffic_VS_active_regions",
# 		variable	=> $VARIABLES{active_regions},
# 		value		=> $VALUES{transmittedBytes},
# 		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
# 		key_position	=> "top left",
# 	},
	{
		id		=> "datarate_VS_active_regions",
		variable	=> $VARIABLES{active_regions},
		value		=> $VALUES{transmittedDatarate},
		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
		key_position	=> "bottom right",
		y_axis_range	=> "[0:525]",
	},
# 	{
# 		id		=> "slp_recall_VS_active_regions",
# 		variable	=> $VARIABLES{active_regions},
# 		value		=> $VALUES{SLP_recall},
# 		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
# 		key_position	=> "bottom left",
# 	},
	{
		id		=> "service_request_delay_VS_active_regions",
		variable	=> $VARIABLES{active_regions},
		value		=> $VALUES{SERVICE_averageSuccessfulDataRequestDelay},
		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
		key_position	=> "top right",
		y_axis_range	=> "[0.01:30]",
	},
	{
		id		=> "service_recall_VS_active_regions",
		variable	=> $VARIABLES{active_regions},
		value		=> $VALUES{SERVICE_requestRecall},
		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
		key_position	=> "bottom left",
	},
	{
		id		=> "service_datarate_VS_active_regions",
		variable	=> $VARIABLES{active_regions},
		value		=> $VALUES{SERVICE_transmittedDatarate},
		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
		key_position	=> "bottom left",
		y_axis_range	=> "[0:105]",
	},
	{
		id		=> "average_service_request_hop_count_VS_active_regions",
		variable	=> $VARIABLES{active_regions},
		value		=> $VALUES{SERVICE_averageRequestHopCount},
		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
		key_position	=> "top right",
		y_axis_range	=> "[0:10.5]",
	},
	{
		id		=> "service_instances_VS_active_regions",
		variable	=> $VARIABLES{active_regions},
		value		=> $VALUES{SERVICE_hostedServices},
		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
		key_position	=> "top right",
		y_axis_range	=> "[0:12.5]",
	},
	{
		id		=> "service_replications_VS_active_regions",
		variable	=> $VARIABLES{active_regions},
		value		=> $VALUES{SERVICE_totalReplications},
		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
		key_position	=> "top left",
		y_axis_range	=> "[0:225]",
	},
);
my @SERVICE_SIZES = (100000);
my @SYNC_RATIO = (1, 2, 4, 8, 16);		# FIXME: Used for ACTIVE_REGIONS.
my @NODES = (100);
# my @ACTIVE_NODE_RATIOS = (1, 2, 3, 4, 6);	# FIXME: Used for ACTIVITY_PHASES.
my @ACTIVE_NODE_RATIOS = (3);			# FIXME: Used for ACTIVITY_PHASES.
# my @REQUEST_FREQUENCIES = ("1.0", "2.0", "5.0");
my @REQUEST_FREQUENCIES = ("5.0");

if(scalar(@ARGV) == 0) {
	generate_summary_plots($EXPERIMENT, \@EVALUATIONS, \@PLACEMENTS, \@SERVICE_SIZES, \@SYNC_RATIO, \@NODES, \@ACTIVE_NODE_RATIOS, \@REQUEST_FREQUENCIES, \%CONTEXT);
}



my @EVALUATIONS = (
# 	{
# 		id		=> "traffic_VS_activity_phases",
# 		variable	=> $VARIABLES{activity_phases},
# 		value		=> $VALUES{transmittedBytes},
# 		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
# 		key_position	=> "top left",
# 	},
	{
		id		=> "datarate_VS_activity_phases",
		variable	=> $VARIABLES{activity_phases},
		value		=> $VALUES{transmittedDatarate},
		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
		key_position	=> "bottom right",
		y_axis_range	=> "[0:525]",
	},
# 	{
# 		id		=> "slp_recall_VS_activity_phases",
# 		variable	=> $VARIABLES{activity_phases},
# 		value		=> $VALUES{SLP_recall},
# 		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
# 		key_position	=> "bottom left",
# 	},
	{
		id		=> "service_request_delay_VS_activity_phases",
		variable	=> $VARIABLES{activity_phases},
		value		=> $VALUES{SERVICE_averageSuccessfulDataRequestDelay},
		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
		key_position	=> "top right",
		y_axis_range	=> "[0.01:30]",
	},
	{
		id		=> "service_recall_VS_activity_phases",
		variable	=> $VARIABLES{activity_phases},
		value		=> $VALUES{SERVICE_requestRecall},
		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
		key_position	=> "bottom left",
	},
	{
		id		=> "service_datarate_VS_activity_phases",
		variable	=> $VARIABLES{activity_phases},
		value		=> $VALUES{SERVICE_transmittedDatarate},
		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
		key_position	=> "bottom left",
		y_axis_range	=> "[0:105]",
	},
	{
		id		=> "average_service_request_hop_count_VS_activity_phases",
		variable	=> $VARIABLES{activity_phases},
		value		=> $VALUES{SERVICE_averageRequestHopCount},
		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
		key_position	=> "top right",
		y_axis_range	=> "[0:10.5]",
	},
	{
		id		=> "service_instances_VS_activity_phases",
		variable	=> $VARIABLES{activity_phases},
		value		=> $VALUES{SERVICE_hostedServices},
		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
		key_position	=> "top right",
		y_axis_range	=> "[0:12.5]",
	},
	{
		id		=> "service_replications_VS_activity_phases",
		variable	=> $VARIABLES{activity_phases},
		value		=> $VALUES{SERVICE_totalReplications},
		plot_type	=> $PLOT_TYPE_DATA_SERIES_PER_PLACEMENT,
		key_position	=> "top left",
		y_axis_range	=> "[0:225]",
	},
);
my @SERVICE_SIZES = (100000);
# my @SYNC_RATIO = (1, 2, 4, 8, 16);		# FIXME: Used for ACTIVE_REGIONS.
my @SYNC_RATIO = (4);				# FIXME: Used for ACTIVE_REGIONS.
my @NODES = (100);
my @ACTIVE_NODE_RATIOS = (1, 2, 3, 4, 6, 8, 10, 12, 15, 20, 30, 60);	# FIXME: Used for ACTIVITY_PHASES.
# my @REQUEST_FREQUENCIES = ("1.0", "2.0", "5.0");
my @REQUEST_FREQUENCIES = ("5.0");

if(scalar(@ARGV) == 0) {
	generate_summary_plots($EXPERIMENT, \@EVALUATIONS, \@PLACEMENTS, \@SERVICE_SIZES, \@SYNC_RATIO, \@NODES, \@ACTIVE_NODE_RATIOS, \@REQUEST_FREQUENCIES, \%CONTEXT);
}



my @EVALUATIONS = (
# 	{
# 		id		=> "packets_VS_hop_count",
# 		variable	=> $VARIABLES{hop_count},
# 		value		=> $VALUES{unicastPacketsPerHopCount},
# 		plot_type	=> $PLOT_TYPE_HOP_COUNT_HISTOGRAM,
# 		key_position	=> "",
# 	},
	{
		id		=> "service_requests_VS_hop_count",
		variable	=> $VARIABLES{hop_count},
		value		=> $VALUES{SERVICE_requestPacketsPerHopCount},
		plot_type	=> $PLOT_TYPE_HOP_COUNT_HISTOGRAM,
		key_position	=> "",
	},
);
my @SERVICE_SIZES = (100000);
my @SYNC_RATIO = (1, 2, 4, 8, 16);		# FIXME: Used for ACTIVE_REGIONS.
my @NODES = (100);
my @ACTIVE_NODE_RATIOS = (1, 3, 6, 9, 12);	# FIXME: Used for ACTIVITY_PHASES.
my @REQUEST_FREQUENCIES = ("5.0");

if(scalar(@ARGV) == 0) {
# 	generate_summary_plots($EXPERIMENT, \@EVALUATIONS, \@PLACEMENTS, \@SERVICE_SIZES, \@SYNC_RATIO, \@NODES, \@ACTIVE_NODE_RATIOS, \@REQUEST_FREQUENCIES, \%CONTEXT);
}



my @EVALUATIONS = (
	{
		id		=> "average_service_request_hop_count_VS_service_instances",
		variable	=> $VALUES{SERVICE_hostedServices},
		value		=> $VALUES{SERVICE_averageRequestHopCount},
		plot_type	=> $PLOT_TYPE_POINTS_PER_PLACEMENT,
		key_position	=> "top right",
		x_axis_range	=> "[0:8.5]",
		y_axis_range	=> "[0:6.25]",
	},
);
my @SERVICE_SIZES = (100000);
my @SYNC_RATIO = (1, 2, 4, 8, 16);		# FIXME: Used for ACTIVE_REGIONS.
my @NODES = (100);
my @ACTIVE_NODE_RATIOS = (1, 3, 6, 9, 12);	# FIXME: Used for ACTIVITY_PHASES.
my @REQUEST_FREQUENCIES = ("5.0");

if(scalar(@ARGV) == 0) {
	generate_summary_plots($EXPERIMENT, \@EVALUATIONS, \@PLACEMENTS, \@SERVICE_SIZES, \@SYNC_RATIO, \@NODES, \@ACTIVE_NODE_RATIOS, \@REQUEST_FREQUENCIES, \%CONTEXT);
}



my @EVALUATIONS = (
	{
		id		=> "datarate_VS_time",
		variable	=> $VARIABLES{current_time},
		value		=> $VALUES{transmittedDatarate},
		plot_type	=> $PLOT_TYPE_TIME_SERIES,
		key_position	=> "",
	},
	{
		id		=> "service_request_delay_VS_time",
		variable	=> $VARIABLES{current_time},
		value		=> $VALUES{SERVICE_averageSuccessfulDataRequestDelay},
		plot_type	=> $PLOT_TYPE_TIME_SERIES,
		key_position	=> "",
	},
	{
		id		=> "service_recall_VS_time",
		variable	=> $VARIABLES{current_time},
		value		=> $VALUES{SERVICE_requestRecall},
		plot_type	=> $PLOT_TYPE_TIME_SERIES,
		key_position	=> "",
	},
	{
		id		=> "service_datarate_VS_time",
		variable	=> $VARIABLES{current_time},
		value		=> $VALUES{SERVICE_transmittedDatarate},
		plot_type	=> $PLOT_TYPE_TIME_SERIES,
		key_position	=> "",
	},
	{
		id		=> "service_instances_VS_time",
		variable	=> $VARIABLES{current_time},
		value		=> $VALUES{SERVICE_hostedServices},
		plot_type	=> $PLOT_TYPE_TIME_SERIES,
		key_position	=> "",
	},
	{
		id		=> "service_replications_VS_time",
		variable	=> $VARIABLES{current_time},
		value		=> $VALUES{SERVICE_totalReplications},
		plot_type	=> $PLOT_TYPE_TIME_SERIES,
		key_position	=> "",
	},
	{
		id		=> "service_migrations_VS_time",
		variable	=> $VARIABLES{current_time},
		value		=> $VALUES{SERVICE_totalMigrations},
		plot_type	=> $PLOT_TYPE_TIME_SERIES,
		key_position	=> "",
	},
	{
		id		=> "service_cost_VS_time",
		variable	=> $VARIABLES{current_time},
		value		=> $VALUES{PLACEMENT_currentServiceCost},
		plot_type	=> $PLOT_TYPE_TIME_SERIES,
		key_position	=> "",
	},
	{
		id		=> "optimal_service_cost_VS_time",
		variable	=> $VARIABLES{current_time},
		value		=> $VALUES{PLACEMENT_currentlyOptimalServiceCost},
		plot_type	=> $PLOT_TYPE_TIME_SERIES,
		key_position	=> "",
	},
	{
		id		=> "optimal_service_instances_VS_time",
		variable	=> $VARIABLES{current_time},
		value		=> $VALUES{PLACEMENT_currentlyOptimalNumberOfHosts},
		plot_type	=> $PLOT_TYPE_TIME_SERIES,
		key_position	=> "",
	},
	{
		id		=> "placement_adaptation_cost_VS_time",
		variable	=> $VARIABLES{current_time},
		value		=> $VALUES{PLACEMENT_currentlyEstimatedAdaptationCost},
		plot_type	=> $PLOT_TYPE_TIME_SERIES,
		key_position	=> "",
	},
	{
		id		=> "placement_adaptations_VS_time",
		variable	=> $VARIABLES{current_time},
		value		=> $VALUES{PLACEMENT_totalAdaptations},
		plot_type	=> $PLOT_TYPE_TIME_SERIES,
		key_position	=> "",
	},
);

if(scalar(@ARGV) > 0) {
	generate_time_plots($EXPERIMENT, \@EVALUATIONS, $ARGV[0]);
}
