##
#
# Generic setup header for all simulations.
#
# Copyright 2007-2008, Freie Universitaet Berlin. All rights reserved.
# Please refer to the COPYING file for detailed licensing information.
#
# Author: Georg Wittenburg
#
# $Id: header.tcl 3084 2009-10-21 15:07:57Z wittenbu $
#
##



# Create God
set god_ [create-god $val(nn)]
$god_ on

# Create simulator
set ns_ [new Simulator]

# Initialize random number generator with new seed (0 -> random seed)
set rng_ [new RNG]
$rng_ seed 0

# Create channel
set chan_ [new $val(chan)]

# Create topography
set topo_ [new Topography]
$topo_ load_flatgrid $val(x) $val(y)

# Setup node configuration
$ns_ node-config \
	-adhocRouting $val(rp) \
	-llType $val(ll) \
	-macType $val(mac) \
	-ifqType $val(ifq) \
	-ifqLen $val(ifqlen) \
	-antType $val(ant) \
	-propType $val(prop) \
	-phyType $val(netif) \
	-topoInstance $topo_ \
	-agentTrace OFF \
	-routerTrace OFF \
	-macTrace OFF \
	-movementTrace OFF \
	-channel $chan_ \
	-energyModel $val(engmodel) \
	-txPower $val(txPower) \
	-rxPower $val(rxPower) \
	-idlePower $val(idlePower) \
	-initialEnergy $val(initeng)

# Enable heuristic random seed in shadowing radio propagation.
if {[$ns_ set propType_] == "Propagation/Shadowing"} then {
	[$ns_ set propInstance_] seed heuristic 0
}



# Initialize exported 802.11 MAC variables
Mac/802_11		set transmittedPackets_				0
Mac/802_11		set transmissionFailures_				0
Mac/802_11		set receivedPackets_				0
Mac/802_11		set droppedPackets_capture_			0
Mac/802_11		set droppedPackets_collision_			0
Mac/802_11		set droppedPackets_transmit_			0
Mac/802_11		set droppedPackets_bit_error_			0
Mac/802_11		set transmittedBytes_				0
Mac/802_11		set receivedBytes_				0

# Initialize exported variables of wrapper application around C code
Application/C		set transmittedUnicastPackets_			0
Application/C		set unicastTransmissionFailures_		0
Application/C		set receivedUnicastPackets_			0
Application/C		set transmittedBroadcastPackets_		0
Application/C		set receivedBroadcastPackets_			0
Application/C		set transmittedBytes_				0
Application/C		set receivedBytes_				0

# Initialize exported C code variables
Application/C		set directNeighbors_				0
Application/C		set longestRoute_				0
Application/C		set averageRoute_				0.0
Application/C		set DEBUG_ERRORs_				0
Application/C		set DEBUG_WARNs_				0
Application/C		set DYMO_transmittedUnicastPackets_		0
Application/C		set DYMO_receivedUnicastPackets_		0
Application/C		set DYMO_transmittedBroadcastPackets_		0
Application/C		set DYMO_receivedBroadcastPackets_		0
Application/C		set RUDP_totalTransmissions_			0
Application/C		set RUDP_successfulTransmissions_		0
Application/C		set ICMP_transmittedNops_			0
Application/C		set ICMP_receivedNops_				0
Application/C		set SLP_totalQueries_				0
Application/C		set SLP_successfulQueries_			0
Application/C		set SERVICE_totalDataRequests_			0
Application/C		set SERVICE_successfulDataRequests_		0
Application/C		set SERVICE_totalSuccessfulDataRequestDelay_	0.0
Application/C		set SERVICE_totalDataUpdates_			0
Application/C		set SERVICE_successfulDataUpdates_		0
Application/C		set SERVICE_totalStreamRequests_		0
Application/C		set SERVICE_successfulStreamRequests_		0
Application/C		set SERVICE_expectedStreamPackets_		0
Application/C		set SERVICE_receivedStreamPackets_		0
Application/C		set SERVICE_successfulReplications_		0
Application/C		set SERVICE_totalMigrations_			0
Application/C		set SERVICE_successfulMigrations_		0
Application/C		set SERVICE_totalReplications_			0
Application/C		set PLACEMENT_total_adaptations_		0
Application/C		set PLACEMENT_successful_adaptations_		0
Application/C		set PLACEMENT_failed_adaptations_		0



# Open trace files
#set tracefd_	[open out.tr w]
set tracefd_	[open /dev/null w]
$ns_ trace-all $tracefd_
#set namtracefd_ [open out.nam w]
set namtracefd_ [open /dev/null w]
$ns_ namtrace-all-wireless $namtracefd_ $val(x) $val(y)

# Set display speed of NAM trace.
$ns_ puts-nam-traceall "v -t 0.0 -e set_rate_ext 200ms 1"

# Remember start time
set start_time_ [clock seconds]

# Begin and end of start, simulation, stop and finalize phases.
set start_phase_begin		0.0
set start_phase_end		[expr $start_phase_begin + $simulation_setup_duration]
set simulation_begin		$start_phase_end
set simulation_end		[expr $simulation_begin + $simulation_duration]
set stop_phase_begin		$simulation_end
set stop_phase_end		[expr $stop_phase_begin + $simulation_setup_duration]
set finalize_phase_begin	[expr $stop_phase_end + 600.0]
set finalize_phase_end		[expr $finalize_phase_begin + 600.0]

for {set t $start_phase_begin} {$t <= $stop_phase_end} {set t [expr $t + 10]} {
	$ns_ at $t "puts [format \"\\\[%.4f\\\]\" $t]"
}

$ns_ at [expr $finalize_phase_end] "stop"



# Calculate the center node for a grid layout.
if {[expr $val(nn) % 2] == 0} {
	set grid_center_node	[expr $val(nn) / 2 - int(sqrt($val(nn))) / 2 - 1]
} else {
	set grid_center_node	[expr $val(nn) / 2]
}



proc stop {} {
	global ns_ tracefd_ namtracefd_ start_time_
	$ns_ halt
	$ns_ nam-end-wireless [$ns_ now]
	$ns_ flush-trace
	close $tracefd_
	close $namtracefd_
	set elapased_time [expr [clock seconds] - $start_time_ - 3600]	;# Formatting miraculously adds one hour.
	puts [format "Simulation complete (%s)." [clock format $elapased_time -format "%T"]]
#	puts ""
#	printProcessStatus
	puts ""
	printStats
	puts "\7"	;# Ring system bell when done.
}

proc printProcessStatus {} {
	set stats [open /proc/self/status r]
	while { [gets $stats line] >= 0 } {
		puts $line
	}
	close $stats
}

# Snippet from http://code.activestate.com/recipes/146220/.
proc thousands {num {sep ,}} {
    while {[regsub {^([-+]?\d+)(\d\d\d)} $num "\\1$sep\\2" num]} {}
    return $num
}

proc printStats {} {
	global val application_ node_

	set MAC_transmittedPackets			0
	set MAC_transmissionFailures			0
	set MAC_receivedPackets				0
	set MAC_droppedPackets_capture			0
	set MAC_droppedPackets_collision		0
	set MAC_droppedPackets_transmit			0
	set MAC_droppedPackets_bit_error		0
	set MAC_transmittedBytes			wide(0)
	set MAC_receivedBytes				wide(0)

	set APP_transmittedUnicastPackets		0
	set APP_unicastTransmissionFailures		0
	set APP_receivedUnicastPackets			0
	set APP_unicastPDR				0.0
	set APP_transmittedBroadcastPackets		0
	set APP_receivedBroadcastPackets		0
	set APP_broadcastPDR				0.0
	set APP_transmittedBytes			wide(0)
	set APP_receivedBytes				wide(0)

	set DEBUG_ERRORs				0
	set DEBUG_WARNs					0

	set minNodeDegree				255
	set maxNodeDegree				0
	set directNeighbors				0
	set avgNodeDegree				0.0
	set directNeighborsList				[list]
	set medNodeDegree				0.0

	set longestRoute				0
	set averageRoute				0.0

	set DYMO_transmittedUnicastPackets		0
	set DYMO_receivedUnicastPackets			0
	set DYMO_unicastPdr				0.0
	set DYMO_transmittedBroadcastPackets		0
	set DYMO_receivedBroadcastPackets		0
	set DYMO_broadcastPdr				0.0

	set RUDP_totalTransmissions			0
	set RUDP_successfulTransmissions		0
	set RUDP_pdr					0.0

	set ICMP_transmittedNops			0
	set ICMP_receivedNops				0
	set ICMP_nopPdr					0.0

	set SLP_totalQueries				0
	set SLP_successfulQueries			0
	set SLP_recall					0.0

	set SERVICE_totalDataRequests			0
	set SERVICE_successfulDataRequests		0
	set SERVICE_dataRequestRecall			0.0
	set SERVICE_totalSuccessfulDataRequestDelay	0.0
	set SERVICE_averageSuccessfulDataRequestDelay	0.0
	set SERVICE_totalDataUpdates			0
	set SERVICE_successfulDataUpdates		0
	set SERVICE_dataUpdateRecall			0.0
	set SERVICE_totalStreamRequests			0
	set SERVICE_successfulStreamRequests		0
	set SERVICE_streamRequestRecall			0.0
	set SERVICE_expectedStreamPackets		0
	set SERVICE_receivedStreamPackets		0
	set SERVICE_streamPacketRecall			0.0
	set SERVICE_totalReplications			0
	set SERVICE_successfulReplications		0
	set SERVICE_replicationSuccessRate		0.0
	set SERVICE_totalMigrations			0
	set SERVICE_successfulMigrations		0
	set SERVICE_migrationSuccessRate		0.0

	set PLACEMENT_totalAdaptations			0
	set PLACEMENT_successfulAdaptations		0
	set PLACEMENT_adaptationSuccessRate		0.0

	for {set i 0} {$i < $val(nn)} {incr i} {
		set MAC_transmittedPackets [expr $MAC_transmittedPackets + [[$node_($i) set mac_(0)] set transmittedPackets_]]
		set MAC_transmissionFailures [expr $MAC_transmissionFailures + [[$node_($i) set mac_(0)] set transmissionFailures_]]
		set MAC_receivedPackets [expr $MAC_receivedPackets + [[$node_($i) set mac_(0)] set receivedPackets_]]
		set MAC_droppedPackets_capture [expr $MAC_droppedPackets_capture + [[$node_($i) set mac_(0)] set droppedPackets_capture_]]
		set MAC_droppedPackets_collision [expr $MAC_droppedPackets_collision + [[$node_($i) set mac_(0)] set droppedPackets_collision_]]
		set MAC_droppedPackets_transmit [expr $MAC_droppedPackets_transmit + [[$node_($i) set mac_(0)] set droppedPackets_transmit_]]
		set MAC_droppedPackets_bit_error [expr $MAC_droppedPackets_bit_error + [[$node_($i) set mac_(0)] set droppedPackets_bit_error_]]
		set MAC_transmittedBytes [expr $MAC_transmittedBytes + [[$node_($i) set mac_(0)] set transmittedBytes_]]
		set MAC_receivedBytes [expr $MAC_receivedBytes + [[$node_($i) set mac_(0)] set receivedBytes_]]

		set APP_transmittedUnicastPackets [expr $APP_transmittedUnicastPackets + [$application_($i) set transmittedUnicastPackets_]]
		set APP_unicastTransmissionFailures [expr $APP_unicastTransmissionFailures + [$application_($i) set unicastTransmissionFailures_]]
		set APP_receivedUnicastPackets [expr $APP_receivedUnicastPackets + [$application_($i) set receivedUnicastPackets_]]
		set APP_transmittedBroadcastPackets [expr $APP_transmittedBroadcastPackets + [$application_($i) set transmittedBroadcastPackets_]]
		set APP_receivedBroadcastPackets [expr $APP_receivedBroadcastPackets + [$application_($i) set receivedBroadcastPackets_]]
		set APP_transmittedBytes [expr $APP_transmittedBytes + [$application_($i) set transmittedBytes_]]
		set APP_receivedBytes [expr $APP_receivedBytes + [$application_($i) set receivedBytes_]]

		set DEBUG_ERRORs [expr $DEBUG_ERRORs + [$application_($i) set DEBUG_ERRORs_]]
		set DEBUG_WARNs [expr $DEBUG_WARNs + [$application_($i) set DEBUG_WARNs_]]

		if {$minNodeDegree > [$application_($i) set directNeighbors_]} then {
			set minNodeDegree [$application_($i) set directNeighbors_]
		}
		if {$maxNodeDegree < [$application_($i) set directNeighbors_]} then {
			set maxNodeDegree [$application_($i) set directNeighbors_]
		}
		set directNeighbors [expr $directNeighbors + [$application_($i) set directNeighbors_]]
		lappend directNeighborsList [$application_($i) set directNeighbors_]

		if {$longestRoute < [$application_($i) set longestRoute_]} then {
			set longestRoute [$application_($i) set longestRoute_]
		}
		set averageRoute [expr $averageRoute + [$application_($i) set averageRoute_]]

		set DYMO_transmittedUnicastPackets [expr $DYMO_transmittedUnicastPackets + [$application_($i) set DYMO_transmittedUnicastPackets_]]
		set DYMO_receivedUnicastPackets [expr $DYMO_receivedUnicastPackets + [$application_($i) set DYMO_receivedUnicastPackets_]]
		set DYMO_transmittedBroadcastPackets [expr $DYMO_transmittedBroadcastPackets + [$application_($i) set DYMO_transmittedBroadcastPackets_]]
		set DYMO_receivedBroadcastPackets [expr $DYMO_receivedBroadcastPackets + [$application_($i) set DYMO_receivedBroadcastPackets_]]

		set RUDP_totalTransmissions [expr $RUDP_totalTransmissions + [$application_($i) set RUDP_totalTransmissions_]]
		set RUDP_successfulTransmissions [expr $RUDP_successfulTransmissions + [$application_($i) set RUDP_successfulTransmissions_]]

		set ICMP_transmittedNops [expr $ICMP_transmittedNops + [$application_($i) set ICMP_transmittedNops_]]
		set ICMP_receivedNops [expr $ICMP_receivedNops + [$application_($i) set ICMP_receivedNops_]]

		set SLP_totalQueries [expr $SLP_totalQueries + [$application_($i) set SLP_totalQueries_]]
		set SLP_successfulQueries [expr $SLP_successfulQueries + [$application_($i) set SLP_successfulQueries_]]

		set SERVICE_totalDataRequests [expr $SERVICE_totalDataRequests + [$application_($i) set SERVICE_totalDataRequests_]]
		set SERVICE_successfulDataRequests [expr $SERVICE_successfulDataRequests + [$application_($i) set SERVICE_successfulDataRequests_]]
		set SERVICE_totalSuccessfulDataRequestDelay [expr $SERVICE_totalSuccessfulDataRequestDelay + [$application_($i) set SERVICE_totalSuccessfulDataRequestDelay_]]
		set SERVICE_totalDataUpdates [expr $SERVICE_totalDataUpdates + [$application_($i) set SERVICE_totalDataUpdates_]]
		set SERVICE_successfulDataUpdates [expr $SERVICE_successfulDataUpdates + [$application_($i) set SERVICE_successfulDataUpdates_]]
		set SERVICE_totalStreamRequests [expr $SERVICE_totalStreamRequests + [$application_($i) set SERVICE_totalStreamRequests_]]
		set SERVICE_successfulStreamRequests [expr $SERVICE_successfulStreamRequests + [$application_($i) set SERVICE_successfulStreamRequests_]]
		set SERVICE_expectedStreamPackets [expr $SERVICE_expectedStreamPackets + [$application_($i) set SERVICE_expectedStreamPackets_]]
		set SERVICE_receivedStreamPackets [expr $SERVICE_receivedStreamPackets + [$application_($i) set SERVICE_receivedStreamPackets_]]
		set SERVICE_totalReplications [expr $SERVICE_totalReplications + [$application_($i) set SERVICE_totalReplications_]]
		set SERVICE_successfulReplications [expr $SERVICE_successfulReplications + [$application_($i) set SERVICE_successfulReplications_]]
		set SERVICE_totalMigrations [expr $SERVICE_totalMigrations + [$application_($i) set SERVICE_totalMigrations_]]
		set SERVICE_successfulMigrations [expr $SERVICE_successfulMigrations + [$application_($i) set SERVICE_successfulMigrations_]]

		set PLACEMENT_totalAdaptations [expr $PLACEMENT_totalAdaptations + [$application_($i) set PLACEMENT_total_adaptations_]]
		set PLACEMENT_successfulAdaptations [expr $PLACEMENT_successfulAdaptations + [$application_($i) set PLACEMENT_successful_adaptations_]]
	}

	set avgNodeDegree [expr double($directNeighbors) / double($val(nn))]
	set directNeighborsList [lsort -integer $directNeighborsList]
	if {[expr [llength $directNeighborsList] % 2] == 0} then {
		set medNodeDegree [expr ([lindex $directNeighborsList [expr [llength $directNeighborsList] / 2 - 1]] + [lindex $directNeighborsList [expr [llength $directNeighborsList] / 2]]) / 2.0]
	} else {
		set medNodeDegree [lindex $directNeighborsList [expr ([llength $directNeighborsList] - 1) / 2]]
	}
	set averageRoute [expr $averageRoute / $val(nn)]

	if {$APP_transmittedUnicastPackets > 0} then {
		set APP_unicastPDR [expr double($APP_receivedUnicastPackets) / double($APP_transmittedUnicastPackets)]
	}
	if {$APP_transmittedBroadcastPackets > 0} then {
		set APP_broadcastPDR [expr double($APP_receivedBroadcastPackets) / double($APP_transmittedBroadcastPackets)]
	}

	if {$DYMO_transmittedUnicastPackets > 0} then {
		set DYMO_unicastPdr [expr double($DYMO_receivedUnicastPackets) / double($DYMO_transmittedUnicastPackets)]
	}
	if {$DYMO_transmittedBroadcastPackets > 0} then {
		set DYMO_broadcastPdr [expr double($DYMO_receivedBroadcastPackets) / double($DYMO_transmittedBroadcastPackets)]
	}

	if {$RUDP_totalTransmissions > 0} then {
		set RUDP_pdr [expr double($RUDP_successfulTransmissions) / double($RUDP_totalTransmissions)]
	}

	if {$ICMP_transmittedNops > 0} then {
		set ICMP_nopPdr [expr double($ICMP_receivedNops) / double($ICMP_transmittedNops)]
	}

	if {$SLP_totalQueries > 0} then {
		set SLP_recall [expr double($SLP_successfulQueries) / double($SLP_totalQueries)]
	}

	if {$SERVICE_totalDataRequests > 0} then {
		set SERVICE_dataRequestRecall [expr double($SERVICE_successfulDataRequests) / double($SERVICE_totalDataRequests)]
	}
	if {$SERVICE_successfulDataRequests > 0} then {
		set SERVICE_averageSuccessfulDataRequestDelay [expr double($SERVICE_totalSuccessfulDataRequestDelay) / double($SERVICE_successfulDataRequests)]
	}
	if {$SERVICE_totalDataUpdates > 0} then {
		set SERVICE_dataUpdateRecall [expr double($SERVICE_successfulDataUpdates) / double($SERVICE_totalDataUpdates)]
	}
	if {$SERVICE_totalStreamRequests > 0} then {
		set SERVICE_streamRequestRecall [expr double($SERVICE_successfulStreamRequests) / double($SERVICE_totalStreamRequests)]
	}
	if {$SERVICE_expectedStreamPackets > 0} then {
		set SERVICE_streamPacketRecall [expr double($SERVICE_receivedStreamPackets) / double($SERVICE_expectedStreamPackets)]
	}
	if {$SERVICE_totalReplications > 0} then {
		set SERVICE_replicationSuccessRate [expr double($SERVICE_successfulReplications) / double($SERVICE_totalReplications)]
	}
	if {$SERVICE_totalMigrations > 0} then {
		set SERVICE_migrationSuccessRate [expr double($SERVICE_successfulMigrations) / double($SERVICE_totalMigrations)]
	}
	if {$PLACEMENT_totalAdaptations > 0} then {
		set PLACEMENT_adaptationSuccessRate [expr double($PLACEMENT_successfulAdaptations) / double($PLACEMENT_totalAdaptations)]
	}

	puts [format "ERRORs:\t\t\t\t\t%d"				$DEBUG_ERRORs]
	puts [format "WARNings:\t\t\t\t%d"				$DEBUG_WARNs]
	puts ""
	puts [format "Nodes:\t\t\t\t\t%d"				$val(nn)]
	puts [format "Area:\t\t\t\t\t%dm x %dm"				$val(x) $val(y)]
	puts [format "Minimal node degree:\t\t\t%d"			$minNodeDegree]
	puts [format "Average node degree:\t\t\t%.2f"			$avgNodeDegree]
	puts [format "Median node degree:\t\t\t%.1f"			$medNodeDegree]
	puts [format "Maximal node degree:\t\t\t%d"			$maxNodeDegree]
	puts [format "Longest route:\t\t\t\t%d hop(s)"			$longestRoute]
	puts [format "Average route:\t\t\t\t%.2f hop(s)"		$averageRoute]
	puts ""
	puts [format "MAC transmitted packets:\t\t%s"			[thousands $MAC_transmittedPackets]]
	puts [format "MAC transmission failures:\t\t%s"			[thousands $MAC_transmissionFailures]]
	puts [format "MAC received packets:\t\t\t%s"			[thousands $MAC_receivedPackets]]
	puts [format "MAC dropped packets (capture):\t\t%s"		[thousands $MAC_droppedPackets_capture]]
	puts [format "MAC dropped packets (collision):\t%s"		[thousands $MAC_droppedPackets_collision]]
	puts [format "MAC dropped packets (transmit):\t\t%s"		[thousands $MAC_droppedPackets_transmit]]
	puts [format "MAC dropped packets (bit error):\t%s"		[thousands $MAC_droppedPackets_bit_error]]
	puts [format "MAC transmitted bytes:\t\t\t%s"			[thousands $MAC_transmittedBytes]]
	puts [format "MAC received bytes:\t\t\t%s"			[thousands $MAC_receivedBytes]]
	puts ""
#	puts [format "APP transmitted unicast packets:\t%s"		[thousands $APP_transmittedUnicastPackets]]
#	puts [format "APP unicast transmission failures:\t%s"		[thousands $APP_unicastTransmissionFailures]]
#	puts [format "APP received unicast packets:\t\t%s"		[thousands $APP_receivedUnicastPackets]]
#	puts [format "APP unicast packet delivery ratio:\t%.2f%%"	[expr $APP_unicastPDR * 100]]
#	puts [format "APP transmitted broadcast packets:\t%s"		[thousands $APP_transmittedBroadcastPackets]]
#	puts [format "APP received broadcast packets:\t\t%s"		[thousands $APP_receivedBroadcastPackets]]
#	puts [format "APP broadcast packet delivery ratio:\t%.2f%%"	[expr $APP_broadcastPDR * 100]]
#	puts [format "APP transmitted bytes:\t\t\t%s"			[thousands $APP_transmittedBytes]]
#	puts [format "APP received bytes:\t\t\t%s"			[thousands $APP_receivedBytes]]
#	puts ""
	puts [format "DYMO transmitted unicast packets:\t%s"		[thousands $DYMO_transmittedUnicastPackets]]
	puts [format "DYMO received unicast packets:\t\t%s"		[thousands $DYMO_receivedUnicastPackets]]
	puts [format "DYMO unicast packet delivery ratio:\t%.2f%%"	[expr $DYMO_unicastPdr * 100]]
	puts [format "DYMO transmitted broadcast packets:\t%s"		[thousands $DYMO_transmittedBroadcastPackets]]
	puts [format "DYMO received broadcast packets:\t%s"		[thousands $DYMO_receivedBroadcastPackets]]
	puts [format "DYMO broadcast packet delivery ratio:\t%.2f%%"	[expr $DYMO_broadcastPdr * 100]]
	puts ""
	puts [format "RUDP total transmissions:\t\t%s"			[thousands $RUDP_totalTransmissions]]
	puts [format "RUDP successful transmissions:\t\t%s"		[thousands $RUDP_successfulTransmissions]]
	puts [format "RUDP packet delivery ratio:\t\t%.2f%%"		[expr $RUDP_pdr * 100]]
	puts ""
	puts [format "ICMP transmitted NOPs:\t\t\t%s"			[thousands $ICMP_transmittedNops]]
	puts [format "ICMP received NOPs:\t\t\t%s"			[thousands $ICMP_receivedNops]]
	puts [format "ICMP NOP packet delivery ratio:\t\t%.2f%%"	[expr $ICMP_nopPdr * 100]]
#	if {$ICMP_transmittedNops > 0} then {
#		for {set i 0} {$i < $val(nn)} {incr i} {
#			puts [format "ICMP NOP packet delivery ratio at node #%d (%.2f, %.2f, %.2f):\t%.2f%%" $i [$node_($i) set X_] [$node_($i) set Y_] [$node_($i) set Z_] [expr double([$application_($i) set ICMP_receivedNops_]) / double($ICMP_transmittedNops) * 100.0]]
#		}
#	}
	puts ""
	puts [format "SLP total queries:\t\t\t%s"			[thousands $SLP_totalQueries]]
	puts [format "SLP successful queries:\t\t\t%s"			[thousands $SLP_successfulQueries]]
	puts [format "SLP recall:\t\t\t\t%.2f%%"			[expr $SLP_recall * 100]]
	puts ""
	puts [format "SERVICE total data requests:\t\t%s"		[thousands $SERVICE_totalDataRequests]]
	puts [format "SERVICE successful data requests:\t%s"		[thousands $SERVICE_successfulDataRequests]]
	puts [format "SERVICE data request recall:\t\t%.2f%%"		[expr $SERVICE_dataRequestRecall * 100]]
	puts [format "SERVICE avg. succ. data request delay:\t%.4fs"	$SERVICE_averageSuccessfulDataRequestDelay]
	puts [format "SERVICE total data updates:\t\t%s"		[thousands $SERVICE_totalDataUpdates]]
	puts [format "SERVICE successful data updates:\t%s"		[thousands $SERVICE_successfulDataUpdates]]
	puts [format "SERVICE data update recall:\t\t%.2f%%"		[expr $SERVICE_dataUpdateRecall * 100]]
#	puts [format "SERVICE total stream requests:\t\t%s"		[thousands $SERVICE_totalStreamRequests]]
#	puts [format "SERVICE successful stream requests:\t%s"		[thousands $SERVICE_successfulStreamRequests]]
#	puts [format "SERVICE stream request recall:\t\t%.2f%%"		[expr $SERVICE_streamRequestRecall * 100]]
#	puts [format "SERVICE expected stream packets:\t%s"		[thousands $SERVICE_expectedStreamPackets]]
#	puts [format "SERVICE received stream packets:\t%s"		[thousands $SERVICE_receivedStreamPackets]]
#	puts [format "SERVICE stream packet recall:\t\t%.2f%%"		[expr $SERVICE_streamPacketRecall * 100]]
	puts ""
	puts [format "SERVICE total replications:\t\t%s"		[thousands $SERVICE_totalReplications]]
	puts [format "SERVICE successful replications:\t%s"		[thousands $SERVICE_successfulReplications]]
	puts [format "SERVICE replications success rate:\t%.2f%%"	[expr $SERVICE_replicationSuccessRate * 100]]
	puts [format "SERVICE total migrations:\t\t%s"			[thousands $SERVICE_totalMigrations]]
	puts [format "SERVICE successful migrations:\t\t%s"		[thousands $SERVICE_successfulMigrations]]
	puts [format "SERVICE migrations success rate:\t%.2f%%"		[expr $SERVICE_migrationSuccessRate * 100]]
	puts ""
	puts [format "PLACEMENT total adaptations:\t\t%s"		[thousands $PLACEMENT_totalAdaptations]]
	puts [format "PLACEMENT successful adaptations:\t%s"		[thousands $PLACEMENT_successfulAdaptations]]
	puts [format "PLACEMENT adaptations success rate:\t%.2f%%"	[expr $PLACEMENT_adaptationSuccessRate * 100]]

	puts ""
	puts "###"
	puts "nodes,area_x,area_y,minNodeDegree,avgNodeDegree,medNodeDegree,maxNodeDegree,longestRoute,averageRoute,MAC_transmittedPackets,MAC_transmissionFailures,MAC_receivedPackets,MAC_droppedPackets_capture,MAC_droppedPackets_collision,MAC_droppedPackets_transmit,MAC_droppedPackets_bit_error,MAC_transmittedBytes,MAC_receivedBytes,APP_transmittedUnicastPackets,APP_unicastTransmissionFailures,APP_receivedUnicastPackets,APP_unicastPDR,APP_transmittedBroadcastPackets,APP_receivedBroadcastPackets,APP_broadcastPDR,APP_transmittedBytes,APP_receivedBytes,DYMO_transmittedUnicastPackets,DYMO_receivedUnicastPackets,DYMO_unicastPdr,DYMO_transmittedBroadcastPackets,DYMO_receivedBroadcastPackets,DYMO_broadcastPdr,RUDP_totalTransmissions,RUDP_successfulTransmissions,RUDP_pdr,ICMP_transmittedNops,ICMP_receivedNops,ICMP_nopPdr,SLP_totalQueries,SLP_successfulQueries,SLP_recall,SERVICE_totalDataRequests,SERVICE_successfulDataRequests,SERVICE_dataRequestRecall,SERVICE_averageSuccessfulDataRequestDelay,SERVICE_totalDataUpdates,SERVICE_successfulDataUpdates,SERVICE_dataUpdateRecall,SERVICE_totalStreamRequests,SERVICE_successfulStreamRequests,SERVICE_streamRequestRecall,SERVICE_expectedStreamPackets,SERVICE_receivedStreamPackets,SERVICE_streamPacketRecall,SERVICE_totalReplications,SERVICE_successfulReplications,SERVICE_replicationSuccessRate,SERVICE_totalMigrations,SERVICE_successfulMigrations,SERVICE_migrationSuccessRate,PLACEMENT_totalAdaptations,PLACEMENT_successfulAdaptations,PLACEMENT_adaptationSuccessRate"
	puts "$val(nn),$val(x),$val(y),$minNodeDegree,$avgNodeDegree,$medNodeDegree,$maxNodeDegree,$longestRoute,$averageRoute,$MAC_transmittedPackets,$MAC_transmissionFailures,$MAC_receivedPackets,$MAC_droppedPackets_capture,$MAC_droppedPackets_collision,$MAC_droppedPackets_transmit,$MAC_droppedPackets_bit_error,$MAC_transmittedBytes,$MAC_receivedBytes,$APP_transmittedUnicastPackets,$APP_unicastTransmissionFailures,$APP_receivedUnicastPackets,$APP_unicastPDR,$APP_transmittedBroadcastPackets,$APP_receivedBroadcastPackets,$APP_broadcastPDR,$APP_transmittedBytes,$APP_receivedBytes,$DYMO_transmittedUnicastPackets,$DYMO_receivedUnicastPackets,$DYMO_unicastPdr,$DYMO_transmittedBroadcastPackets,$DYMO_receivedBroadcastPackets,$DYMO_broadcastPdr,$RUDP_totalTransmissions,$RUDP_successfulTransmissions,$RUDP_pdr,$ICMP_transmittedNops,$ICMP_receivedNops,$ICMP_nopPdr,$SLP_totalQueries,$SLP_successfulQueries,$SLP_recall,$SERVICE_totalDataRequests,$SERVICE_successfulDataRequests,$SERVICE_dataRequestRecall,$SERVICE_averageSuccessfulDataRequestDelay,$SERVICE_totalDataUpdates,$SERVICE_successfulDataUpdates,$SERVICE_dataUpdateRecall,$SERVICE_totalStreamRequests,$SERVICE_successfulStreamRequests,$SERVICE_streamRequestRecall,$SERVICE_expectedStreamPackets,$SERVICE_receivedStreamPackets,$SERVICE_streamPacketRecall,$SERVICE_totalReplications,$SERVICE_successfulReplications,$SERVICE_replicationSuccessRate,$SERVICE_totalMigrations,$SERVICE_successfulMigrations,$SERVICE_migrationSuccessRate,$PLACEMENT_totalAdaptations,$PLACEMENT_successfulAdaptations,$PLACEMENT_adaptationSuccessRate"
}
