##
#
# Evaluation of service instances and related cost.
#
# Copyright 2007-2009, Freie Universitaet Berlin. All rights reserved.
# Please refer to the COPYING file for detailed licensing information.
#
# Author: Georg Wittenburg
#
# $Id: test.tcl 2896 2009-08-20 17:45:16Z wittenbu $
#
##



source common/config.tcl

# Number of mobile nodes.
set val(nn)	100

# Size of topography (m).
set val(x)	1000
set val(y)	1000

# Simulation setup duration (s).
#set simulation_setup_duration	60.0	;# 1 minute
#set simulation_setup_duration	300.0	;# 5 minutes
set simulation_setup_duration	900.0	;# 15 minutes
#set simulation_setup_duration	3600.0	;# 1 hour

# Simulation duration (s).
set simulation_duration	60.0	;# 1 minute
#set simulation_duration	300.0	;# 5 minutes
#set simulation_duration	900.0	;# 15 minutes
#set simulation_duration	3600.0	;# 1 hour



source common/header.tcl

# Create, initialize and connect nodes, agents and applications.
for {set i 0} {$i < $val(nn)} {incr i} {
	set node_($i) [$ns_ node]
#	$node_($i) set X_ [$rng_ uniform 0 $val(x)]
#	$node_($i) set Y_ [$rng_ uniform 0 $val(y)]
	$node_($i) set X_ [expr ($i % int(sqrt($val(nn)))) * $grid_spacing_distance]
	$node_($i) set Y_ [expr ($i / int(sqrt($val(nn)))) * $grid_spacing_distance]
	$node_($i) set Z_ 0
	$node_($i) random-motion 0		;# Disable random motion
	$god_ new_node $node_($i)
	$ns_ initial_node_pos $node_($i) 10
	puts [format "Placing node #%d at (%.2f, %.2f, %.2f)..." $i [$node_($i) set X_] [$node_($i) set Y_] [$node_($i) set Z_]]

	set agent_($i) [new Agent/MessagePassing]
	set application_($i) [new Application/C]
	$node_($i) attach $agent_($i)
	$application_($i) attach-agent $agent_($i)
	$application_($i) seed heuristic 0	;# Enable heuristic random seed in application

	set start_time($i) [$rng_ uniform $start_phase_begin [expr $start_phase_end / 2]]
#	set stop_time($i) [$rng_ uniform $stop_phase_begin $stop_phase_end]
	set stop_time($i) $stop_phase_begin
	$ns_ at $start_time($i) "$application_($i) start"
	$ns_ at $stop_time($i) "$application_($i) stop"
	$ns_ at $finalize_phase_begin "$application_($i) finalize"

	$application_($i) set_number_of_nodes $val(nn)
	$application_($i) PLACEMENT_set_algorithm none
#	$ns_ at $simulation_begin "$application_($i) PLACEMENT_set_algorithm simple_heuristic"
#	$ns_ at $simulation_begin "$application_($i) PLACEMENT_set_algorithm graph_cost_single_instance"
	$ns_ at $simulation_begin "$application_($i) PLACEMENT_set_algorithm graph_cost_multiple_instances"
#	$ns_ at $simulation_begin "$application_($i) PLACEMENT_set_algorithm linkpull_migration"
#	$ns_ at $simulation_begin "$application_($i) PLACEMENT_set_algorithm peerpull_migration"
#	$ns_ at $simulation_begin "$application_($i) PLACEMENT_set_algorithm tree-topology_migration"
	$ns_ at $simulation_end "$application_($i) PLACEMENT_set_algorithm none"
}
puts [format "Center node is node #%d at (%.2f, %.2f, %.2f)." $grid_center_node [$node_($grid_center_node) set X_] [$node_($grid_center_node) set Y_] [$node_($grid_center_node) set Z_]]



#
# Service setup
#

# Location factor (0.0 -> worst; 1.0 -> best).
set location_factor 1.0

# Service requests per node per second.
set request_frequency 1.0

# Active node ratio (0.0 -> all nodes quiet; 1.0 -> all nodes active).
set activity_factor 1.0

# Place node #0 relative to the middle of the simulation area.
#$node_(0) set X_ [expr $val(x) / 2 * $location_factor]
#$node_(0) set Y_ [expr $val(y) / 2 * $location_factor]

# Start initial service instance of "srv_A".
$ns_ at 0 "$application_($grid_center_node) SERVICE_create_service srv_A 4096 %%sync_traffic_ratio%%"



#
# Service requests
#

# Have all active nodes in the network request data of "srv_A".
for {set i 0} {$i < [expr $val(nn) * $activity_factor]} {incr i} {
	for {set t $start_time($i)} {$t < $stop_time($i)} {set t [expr $t + (1.0 / $request_frequency)]} {
		set t_rand [$rng_ uniform $t [expr $t + (1.0 / $request_frequency)]]
		$ns_ at $t_rand "$application_($i) SERVICE_request_data srv_A 0x0000 1024"
#		puts [format "Scheduling service request from node #%d at %.4f..." $i $t_rand]
	}
}



# Start simulation.
puts "Starting Simulation..."
$ns_ run
