#!/usr/bin/perl


my $eps_file = "service_instances_and_cost_curve.eps";
my $pdf_file = "service_instances_and_cost_curve.pdf";

print "Generating $pdf_file...\n";
open(GNUPLOT, "| gnuplot") or die "can't open gnuplot: $!";

print GNUPLOT <<EOF or die "can't print to gnuplot: $!";
#
# Output control
#

set terminal postscript eps enhanced color 24
#set terminal postscript eps enhanced monochrome 24 blacktext
set output "$eps_file"



#
# Input control
#

set datafile separator ","



#
# Scale / aspect ratio
#

set size ratio 0.5 1.0,1.6180339887	# Golden ratio



#
# Axis: labels, ranges, scales
#

#set xlabel "# Service Instances |{/Helvetica-Italic H_s}|"
set xlabel "# Service Instances"
set xrange [1:100]
set logscale x
set ylabel "Service Cost (kB/s)" 2
set yrange [0:1000]
#set logscale y

#
# Legend
#

#set key top right
set key top left
#set key bottom left
#set key bottom right
#set key outside
set key spacing 0.9
set key width -2.5
#set key height 1
set key samplen 2
#set key box



#
# Line and point styles
#

set style line 1 lt 2 lc rgbcolor "forest-green" lw 2 pt 1
set style line 2 lt 2 lc rgbcolor "green" lw 4 pt 2
set style line 3 lt 2 lc rgbcolor "yellow" lw 2 pt 3
set style line 4 lt 2 lc rgbcolor "orange" lw 2 pt 4
set style line 5 lt 2 lc rgbcolor "dark-orange" lw 2 pt 5
set style line 6 lt 2 lc rgbcolor "red" lw 2 pt 6



#
# Plot
#

#sync_traffic_ratio(1),service_instances(2),cost_MEAN(3),cost_MEDIAN(4),cost_MIN(5),cost_MAX(6),cost_VAR(7),cost_STDDEV(8)

set style data errorlines
#set style data errorbars
plot \\
"< egrep '^0.00,' service_instances_and_cost_curve_STATS.csv" using 2:(\$4/1000):(\$5/1000):(\$6/1000) ls 1 title "{/Helvetica=16 0% sync}", \\
"< egrep '^0.01,' service_instances_and_cost_curve_STATS.csv" using 2:(\$4/1000):(\$5/1000):(\$6/1000) ls 2 title "{/Helvetica=16 1% sync}", \\
"< egrep '^0.02,' service_instances_and_cost_curve_STATS.csv" using 2:(\$4/1000):(\$5/1000):(\$6/1000) ls 3 title "{/Helvetica=16 2% sync}", \\
"< egrep '^0.05,' service_instances_and_cost_curve_STATS.csv" using 2:(\$4/1000):(\$5/1000):(\$6/1000) ls 4 title "{/Helvetica=16 3% sync}", \\
"< egrep '^0.10,' service_instances_and_cost_curve_STATS.csv" using 2:(\$4/1000):(\$5/1000):(\$6/1000) ls 5 title "{/Helvetica=16 10% sync}", \\
"< egrep '^0.20,' service_instances_and_cost_curve_STATS.csv" using 2:(\$4/1000):(\$5/1000):(\$6/1000) ls 6 title "{/Helvetica=16 20% sync}"

EOF

while(<GNUPLOT>) {print;}
close GNUPLOT;
print `ps2pdf $eps_file`;
print `pdfcrop $pdf_file $pdf_file > /dev/null`;
unlink $eps_file;
