##
#
# Routing evaluation script.
#
# Copyright (c) 2007, 2008, 2009, 2010 Georg Wittenburg, Freie Universitaet
# Berlin (wittenbu@inf.fu-berlin.de). All rights reserved.
# Please refer to the COPYING file for detailed licensing information.
#
# Author: Georg Wittenburg
#
# $Id: routing_evaluation.tcl 3237 2010-01-24 15:36:41Z wittenbu $
#
##



source common/config.tcl

# Number of mobile nodes.
set val(nn)	100

# Size of topography (m).
#set val(x)	1000
set val(x)	[expr int(ceil(sqrt($val(nn)))) * $grid_spacing_distance]
#set val(y)	1000
set val(y)	[expr int(ceil(sqrt($val(nn)))) * $grid_spacing_distance]

# Simulation setup duration (s).
#set simulation_setup_duration	0.0	;# 0 minutes
set simulation_setup_duration	60.0	;# 1 minute
#set simulation_setup_duration	300.0	;# 5 minutes
#set simulation_setup_duration	900.0	;# 15 minutes
#set simulation_setup_duration	3600.0	;# 1 hour

# Simulation duration (s).
set simulation_duration	60.0	;# 1 minute
#set simulation_duration	300.0	;# 5 minutes
#set simulation_duration		900.0	;# 15 minutes
#set simulation_duration	3600.0	;# 1 hour



source common/header.tcl

# Create, initialize and connect nodes, agents and applications.
for {set i 0} {$i < $val(nn)} {incr i} {
	set node_($i) [$ns_ node]
#	$node_($i) set X_ $i
	$node_($i) set X_ [$rng_ uniform 0 $val(x)]
#	$node_($i) set X_ [expr ($i % int(sqrt($val(nn)))) * $grid_spacing_distance]
#	$node_($i) set Y_ 0
	$node_($i) set Y_ [$rng_ uniform 0 $val(y)]
#	$node_($i) set Y_ [expr ($i / int(sqrt($val(nn)))) * $grid_spacing_distance]
	$node_($i) set Z_ 0
	$node_($i) random-motion 0		;# Disable random motion
	$god_ new_node $node_($i)
	$ns_ initial_node_pos $node_($i) 10
	puts [format "Placing node #%d at (%.2f, %.2f, %.2f)..." $i [$node_($i) set X_] [$node_($i) set Y_] [$node_($i) set Z_]]

	set agent_($i) [new Agent/MessagePassing]
	set application_($i) [new Application/C]
	$node_($i) attach $agent_($i)
	$application_($i) attach-agent $agent_($i)
	$application_($i) seed heuristic 0	;# Enable heuristic random seed in application

	set start_time($i) [$rng_ uniform $start_phase_begin $start_phase_end]
#	set stop_time($i) [$rng_ uniform $stop_phase_begin $stop_phase_end]
	set stop_time($i) $stop_phase_begin
	$ns_ at $start_time($i) "$application_($i) start"
	$ns_ at $stop_time($i) "$application_($i) stop"
	$ns_ at $finalize_phase_begin "$application_($i) finalize"
	$ns_ at $simulation_end "$application_($i) SERVICE_set_placement_algorithm none"
}
#puts [format "Center node is node #%d at (%.2f, %.2f, %.2f)." $grid_center_node [$node_($grid_center_node) set X_] [$node_($grid_center_node) set Y_] [$node_($grid_center_node) set Z_]]

if {0.0 != 0.0} {
	set mobility_file [format "duration_%s_nodes_%s_speed_%s_run_%s.tcl" 960 100 0.5 01]
	puts [format "Using mobility file %s..." $mobility_file]
	source mobility/$mobility_file
}



#
# Routing evaluation.
#

# Send a NOP from node #0 to node #1.
#$ns_ at $simulation_begin "$application_(0) ICMP_send_nop 1 20 1024"

# Packets per node per second.
set packet_frequency 1.0

# Send packet_frequency NOPs per second from all nodes to the center node.
#set destination_node $grid_center_node
set destination_node [$rng_ integer $val(nn)]
for {set i 0} {$i < $val(nn)} {incr i} {
	for {set t $start_time($i)} {$t < $stop_time($i)} {set t [expr $t + (1.0 / $packet_frequency)]} {
		$ns_ at [$rng_ uniform $t [expr $t + (1.0 / $packet_frequency)]] "$application_($i) ICMP_send_nop $destination_node 20 1024"
	}
}

# Send packet_frequency NOPs per second from all nodes to a random other nodes.
#for {set i 0} {$i < $val(nn)} {incr i} {
#	for {set t $start_time($i)} {$t < $stop_time($i)} {set t [expr $t + (1.0 / $packet_frequency)]} {
#		set random_node [$rng_ integer $val(nn)]
#		$ns_ at [$rng_ uniform $t [expr $t + (1.0 / $packet_frequency)]] "$application_($i) ICMP_send_nop $random_node 20 1024"
#	}
#}



# Start simulation.
puts "Starting Simulation..."
$ns_ run
