##
#
# Test script.
#
# Copyright (c) 2007, 2008, 2009, 2010 Georg Wittenburg, Freie Universitaet
# Berlin (wittenbu@inf.fu-berlin.de). All rights reserved.
# Please refer to the COPYING file for detailed licensing information.
#
# Author: Georg Wittenburg
#
# $Id: test.tcl 3719 2010-08-23 13:00:36Z gwitten $
#
##



source common/config.tcl

# Number of mobile nodes.
set val(nn)	100

# Size of topography (m) is initialized while setting up nodes.
#set val(x)	1000
set val(x)	[expr int(ceil(sqrt($val(nn)))) * $grid_spacing_distance]
#set val(y)	1000
set val(y)	[expr int(ceil(sqrt($val(nn)))) * $grid_spacing_distance]

# Simulation setup duration (s).
#set simulation_setup_duration	0.0	;# 0 minutes
#set simulation_setup_duration	60.0	;# 1 minute
#set simulation_setup_duration	120.0	;# 2 minutes
set simulation_setup_duration	300.0	;# 5 minutes
#set simulation_setup_duration	900.0	;# 15 minutes
#set simulation_setup_duration	3600.0	;# 1 hour

# Simulation duration (s).
#set simulation_duration	0.0	;# 0 minute
#set simulation_duration	60.0	;# 1 minute
#set simulation_duration	300.0	;# 5 minutes
#set simulation_duration	480.0	;# 8 minutes
set simulation_duration		900.0	;# 15 minutes
#set simulation_duration	3600.0	;# 1 hour



source common/header.tcl

# Create, initialize and connect nodes, agents and applications.
for {set i 0} {$i < $val(nn)} {incr i} {
	set node_($i) [$ns_ node]
#	$node_($i) set X_ $i
	$node_($i) set X_ [$rng_ uniform 0 $val(x)]
#	$node_($i) set X_ [expr ($i % int(sqrt($val(nn)))) * $grid_spacing_distance]
#	$node_($i) set Y_ 0
	$node_($i) set Y_ [$rng_ uniform 0 $val(y)]
#	$node_($i) set Y_ [expr ($i / int(sqrt($val(nn)))) * $grid_spacing_distance]
	$node_($i) set Z_ 0
	$node_($i) random-motion 0		;# Disable random motion
	$god_ new_node $node_($i)
	$ns_ initial_node_pos $node_($i) 10
	puts [format "Placing node #%d at (%.2f, %.2f, %.2f)..." $i [$node_($i) set X_] [$node_($i) set Y_] [$node_($i) set Z_]]

	set agent_($i) [new Agent/MessagePassing]
	set application_($i) [new Application/C]
	$node_($i) attach $agent_($i)
	$application_($i) attach-agent $agent_($i)
	$application_($i) seed heuristic 0	;# Enable heuristic random seed in application

	set start_time($i) [$rng_ uniform $start_phase_begin $start_phase_end]
#	set stop_time($i) [$rng_ uniform $stop_phase_begin $stop_phase_end]
	set stop_time($i) $stop_phase_begin
	$ns_ at $start_time($i) "$application_($i) start"
	$ns_ at $stop_time($i) "$application_($i) stop"
	$ns_ at $finalize_phase_begin "$application_($i) finalize"
	$ns_ at $simulation_end "$application_($i) SERVICE_set_placement_algorithm none"
}
#puts [format "Center node is node #%d at (%.2f, %.2f, %.2f)." $grid_center_node [$node_($grid_center_node) set X_] [$node_($grid_center_node) set Y_] [$node_($grid_center_node) set Z_]]



#
# Diagnostics and graph construction.
#

# Send a NOP from node #0 to node #1.
#$ns_ at $simulation_begin "$application_(0) ICMP_send_nop 1"

# Send several NOPs to node #9.
#for {set i 1} {$i <= 1} {incr i} {
#	$ns_ at [$rng_ uniform $simulation_begin $simulation_end] "$application_(0) ICMP_send_nop 9"
#}

# Broadcast a NOP to all nodes in the network.
#$ns_ at $simulation_begin "$application_(0) ICMP_send_nop"

# Broadcast an echo request to all nodes in the one-hop neighborhood.
#$ns_ at $simulation_begin "$application_(0) ICMP_send_local_echo_request"

# Broadcast an echo request to all nodes in the network.
#$ns_ at $simulation_begin "$application_(0) ICMP_send_echo_request"

# Broadcast an echo request from all nodes to all nodes in the network.
#for {set i 0} {$i < $val(nn)} {incr i} {
#	$ns_ at [$rng_ uniform $simulation_begin $simulation_end] "$application_($i) ICMP_send_local_echo_request"
#	$ns_ at [$rng_ uniform $simulation_begin $simulation_end] "$application_($i) ICMP_send_echo_request"
#}

# Construct network graphs with node #0 as coordinator.
#$ns_ at $simulation_begin "$application_(15) INTERFERENCE_measure"



#
# Service setup.
#

# Location factor (0.0 -> worst; 1.0 -> best).
set location_factor 1.0

# Service requests per node per second.
set request_frequency 1.0

# Active node ratio (0.0 -> all nodes quiet; 1.0 -> all nodes active).
set active_node_ratio 0.5

# Synchronization ratio (0.0 -> no synchronization traffic; 1.0 -> synchronization of every request).
set sync_ratio 0.050

# Node to host the first service instance.
#set initial_service_host 0
#set initial_service_host $grid_center_node
set initial_service_host [$rng_ integer $val(nn)]

# Place node #0 relative to the middle of the simulation area.
#$node_(0) set X_ [expr $val(x) / 2 * $location_factor]
#$node_(0) set Y_ [expr $val(y) / 2 * $location_factor]

# Place node #1 in the middle of the simulation area.
#$node_(1) set X_ [expr $val(x) / 2]
#$node_(1) set Y_ [expr $val(y) / 2]

# Send a ICMP NOP to all nodes in the 1-hop neighborhood of node #0.
#for {set t $simulation_begin} {$t < $simulation_end} {set t [expr $t + (1.0 / $request_frequency)]} {
#	$ns_ at 1 "$application_(0) ICMP_send_nop -1 0"
#}

# Start initial service instance of "srvA".
$ns_ at 0 "$application_($initial_service_host) SERVICE_create_service srvA 100000 $sync_ratio"
#$ns_ at 0 "$application_($initial_service_host) SERVICE_create_service srvA 10000 $sync_ratio"
#$ns_ at 1 "$application_($initial_service_host) SERVICE_is_local_service srvA"

# Start placement algorithm.
#$ns_ at 0 "$application_($initial_service_host) SERVICE_set_placement_algorithm srvA none"
#$ns_ at 0 "$application_($initial_service_host) SERVICE_set_placement_algorithm srvA simple_heuristic"
#$ns_ at 0 "$application_($initial_service_host) SERVICE_set_placement_algorithm srvA graph_cost_single_instance"
#$ns_ at 0 "$application_($initial_service_host) SERVICE_set_placement_algorithm srvA graph_cost_multiple_instances"
#$ns_ at 0 "$application_($initial_service_host) SERVICE_set_placement_algorithm srvA linkpull_migration"
#$ns_ at 0 "$application_($initial_service_host) SERVICE_set_placement_algorithm srvA peerpull_migration"
#$ns_ at 0 "$application_($initial_service_host) SERVICE_set_placement_algorithm srvA topocenter_migration"
$ns_ at 0 "$application_($initial_service_host) SERVICE_set_placement_algorithm srvA tree-topology_migration"
#$ns_ at 0 "$application_($initial_service_host) SERVICE_set_placement_algorithm srvA asg_simple"
#$ns_ at 0 "$application_($initial_service_host) SERVICE_set_placement_algorithm srvA asg_event_flow_tree"

# Shutdown initial service host.
#$ns_ at 60 "$node_($initial_service_host) setenergy 0.0"
#$ns_ at 60 "$application_($initial_service_host) stop"
#$ns_ at 60 "$application_($initial_service_host) finalize"



# Have node #1 locate service "srvA"
#$ns_ at $simulation_begin "$application_(9) SLP_locate_service srvA"

# Have all nodes in the network locate the service "srvA".
#for {set i 1} {$i < $val(nn)} {incr i} {
#	$ns_ at [$rng_ uniform $simulation_begin $simulation_end] "$application_($i) SLP_locate_service srvA"
#}



#
# Service requests and updates.
#

# Request one data item from of "srvA".
#$ns_ at $simulation_begin "$application_(0) SERVICE_request_data srvA 0x0000 1024"

# Have one node network request data of "srvA".
#for {set t $simulation_begin} {$t < [expr $simulation_end]} {set t [expr $t + (1.0 / $request_frequency)]} {
#	set t_rand [$rng_ uniform $t [expr $t + (1.0 / $request_frequency)]]
#	$ns_ at $t_rand "$application_(1) SERVICE_request_data srvA 0x0000 1024"
#}


set active_nodes [list 0 1 2 7 8 9 14 15 16 32 33 34 39 40 41 46 47 48]

# Have all active nodes in the network request or update data of "srvA".
for {set i 0} {$i < $val(nn)} {incr i} {
	if {[$rng_ uniform 0.0 1.0] <= $active_node_ratio} {
#	if { [lsearch -exact $active_nodes $i] != -1 } {}
		for {set t $start_time($i)} {$t < [expr $stop_time($i) - (1.0 / $request_frequency)]} {set t [expr $t + (1.0 / $request_frequency)]} {
			set t_rand [$rng_ uniform $t [expr $t + (1.0 / $request_frequency)]]
			if {[$rng_ uniform 0.0 1.0] <= $sync_ratio} {
				$ns_ at $t_rand "$application_($i) SERVICE_update_data srvA 0x0000 1024"
				#$ns_ at $t_rand "$application_($i) SERVICE_update_data srvA 0x0000 256"
				#puts [format "Scheduling service update from node #%d at %.4f..." $i $t_rand]
			} else {
				$ns_ at $t_rand "$application_($i) SERVICE_request_data srvA 0x0000 1024"
				#$ns_ at $t_rand "$application_($i) SERVICE_request_data srvA 0x0000 256"
				#puts [format "Scheduling service request from node #%d at %.4f..." $i $t_rand]
			}
		}
	}
}

# Update one data item of "srvA".
#$ns_ at $simulation_begin "$application_(1) SERVICE_update_data srvA 0x0000 1024"

# Have all nodes in the network update data of "srvA".
#for {set t $simulation_begin} {$t < [expr $simulation_end]} {set t [expr $t + (1.0 / $request_frequency)]} {
#	for {set i 0} {$i < $val(nn)} {incr i} {
#		set t_rand [$rng_ uniform $t [expr $t + (1.0 / $request_frequency)]]
#		$ns_ at $t_rand "$application_($i) SERVICE_update_data srvA 0x0000 1024"
#	}
#}

# Request a stream of "srvA".
#$ns_ at $simulation_begin "$application_(1) SERVICE_request_stream srvA 0x0000 1024 10.0 30.0"

# Have all nodes in the network request a stream of "srvA".
#for {set i 0} {$i < $val(nn)} {incr i} {
#	set t_rand [$rng_ uniform $simulation_begin [expr $simulation_begin + (1.0 / $request_frequency)]]
#	$ns_ at $t_rand "$application_($i) SERVICE_request_stream srvA 0x0000 1024 10.0 [expr $simulation_duration - (1.0 / $request_frequency)]"
#}



#
# Service replication and migration.
#

# Replicate service "srvA" from node #0 on node #1.
#$ns_ at [expr $simulation_begin + ($simulation_duration / 2)] "$application_(0) SERVICE_replicate_service srvA 1"

# Migrate service "srvA" from node #0 to node #1.
#$ns_ at [expr $simulation_begin + ($simulation_duration / 2)] "$application_(0) SERVICE_migrate_service srvA 1"



# Start simulation.
puts "Starting Simulation..."
$ns_ run
