/**
 * @file dymo.h
 *
 * @brief Dynamic MANET On-demand (DYMO) routing protocol (header).
 *
 * Copyright (c) 2007, 2008, 2009, 2010 Georg Wittenburg, Freie Universitaet
 * Berlin (wittenbu@inf.fu-berlin.de). All rights reserved.
 * Please refer to the COPYING file for detailed licensing information.
 *
 * @author Georg Wittenburg
 *
 * $Id: dymo.h 3275 2010-02-08 22:42:58Z gwitten $
 */

#ifndef DYMO_H
#define DYMO_H

#include "../types.h"
#include "dymo_types.h"
#include "dymo_vars.h"

// Interface to main module.
void DYMO_init();
void DYMO_start();
void DYMO_stop();
void DYMO_finalize();
void DYMO_recv(const net_header_t* header, const void* payload);
bool DYMO_command(int argc, const char*const* argv);

// Interface provided by this module.
void DYMO_route(net_header_t* header, const void* payload);
void DYMO_route_recv(const net_header_t* header, const void* payload);
void DYMO_handle_unreachable_next_hop(C_addr_t next_hop_address);
size_t DYMO_max_payload_size();

#endif	// DYMO_H
