/**
 * @file gcmi_clustering.h
 *
 * @brief Graph cost multiple instances (clustering / header).
 *
 * Copyright (c) 2007, 2008, 2009, 2010 Georg Wittenburg, Freie Universitaet
 * Berlin (wittenbu@inf.fu-berlin.de). All rights reserved.
 * Please refer to the COPYING file for detailed licensing information.
 *
 * @author Georg Wittenburg
 *
 * $Id: gcmi_clustering.h 3177 2010-01-05 16:43:45Z wittenbu $
 */

#ifndef GCMI_CLUSTERING_H
#define GCMI_CLUSTERING_H

#include "../../types.h"
#include "../../service/service_types.h"



#ifdef C_SERVICE_PLACEMENT_EVALUATION



/**
 * @brief Cluster of nodes with cluster head.
 */
typedef struct {
	SERVICE_host_stat_t*	host_stat;		///< Pointer to host stat of the cluster head.
	list_t*			node_stats;		///< Pointer to list of node stats of nodes in this cluster.
	list_t*			adjacent_clusters;	///< Pointer to list of adjacent clusters.
} cluster_t;



void cluster_init(cluster_t** cluster, SERVICE_node_stat_t* head_node_stat, list_t* clusters, cluster_t** node_id_to_cluster_map);
void cluster_free_list(void* p);
bool cluster_add_node(cluster_t* cluster, SERVICE_node_stat_t* node_stat_to_add, list_t* clusters, cluster_t** node_id_to_cluster_map);
void cluster_consolidate(cluster_t* changed_cluster, list_t* clusters, cluster_t** node_id_to_cluster_map);
void cluster_merge(cluster_t* dst_cluster, cluster_t* src_cluster, list_t* clusters, cluster_t** node_id_to_cluster_map);
void cluster_to_list_of_head_addresses(const list_t* clusters, list_t* cluster_head_addresses);
void print_clusters(uint8_t debug_level, uint8_t msg_type, list_t* clusters);

#endif	// C_SERVICE_PLACEMENT_EVALUATION

#endif	// GCMI_CLUSTERING_H
