/**
 * @file placement_vars.h
 *
 * @brief Placement (variables).
 *
 * Copyright (c) 2007, 2008, 2009, 2010 Georg Wittenburg, Freie Universitaet
 * Berlin (wittenbu@inf.fu-berlin.de). All rights reserved.
 * Please refer to the COPYING file for detailed licensing information.
 *
 * @author Georg Wittenburg
 *
 * $Id: placement_vars.h 3220 2010-01-18 23:08:26Z wittenbu $
 */

#ifndef PLACEMENT_VARS_H
#define PLACEMENT_VARS_H

#include "../types.h"
#include "placement_types.h"



/**
 * @brief Pointer to the currently outgoing adaptation.
 *
 * We need a global variable because the ADAPTATION COMPLETE message is
 * asynchronous.
 */
PLACEMENT_outgoing_adaptation_t* C_PLACEMENT_currently_outgoing_adaptation;

/**
 * @brief Pointer to list of services to shutdown after a delay.
 */
list_t* C_PLACEMENT_services_to_shutdown_after_delay;

/**
 * @brief Algorithm-specific cost of the current service configuration.
 */
double C_PLACEMENT_current_service_cost;

/**
 * @brief Algorithm-specific cost of the currently optimal service configuration.
 */
double C_PLACEMENT_currently_optimal_service_cost;

/**
 * @brief Number of hosts of the currently optimal service configuration.
 */
uint32_t C_PLACEMENT_currently_optimal_number_of_hosts;

/**
 * @brief Algorithm-specific estimated cost of an adaptation from the current
 * to the optimal service configuration.
 */
double C_PLACEMENT_currently_estimated_adaptation_cost;

/**
 * @brief Statistics counter for total number of adaptations.
 */
uint32_t C_PLACEMENT_total_adaptations;

/**
 * @brief Statistics counter for number of successful adaptations.
 */
uint32_t C_PLACEMENT_successful_adaptations;

/**
 * @brief Statistics counter for number of failed adaptations.
 */
uint32_t C_PLACEMENT_failed_adaptations;

#endif	// PLACEMENT_VARS_H
