/**
 * @file rudp-light.h
 *
 * @brief Reliable User Datagram Protocol (RUDP) - minimal implemenation (header).
 *
 * Copyright (c) 2007, 2008, 2009, 2010 Georg Wittenburg, Freie Universitaet
 * Berlin (wittenbu@inf.fu-berlin.de). All rights reserved.
 * Please refer to the COPYING file for detailed licensing information.
 *
 * @author Georg Wittenburg
 *
 * $Id: rudp-light.h 3275 2010-02-08 22:42:58Z gwitten $
 */

#ifndef RUDP_LIGHT_H
#define RUDP_LIGHT_H

#include "../types.h"
#include "rudp-light_types.h"
#include "rudp-light_vars.h"

#define RUDP_PACKET_TYPE_DATA	0x0	///< RUDP DATA packet type.
#define RUDP_PACKET_TYPE_ACK	0x1	///< RUDP ACK packet type.

// Interface to main module.
void RUDP_init();
void RUDP_start();
void RUDP_stop();
void RUDP_finalize();
void RUDP_recv(const net_header_t* header, const void* payload);
bool RUDP_command(int argc, const char*const* argv);

// Interface provided by this module.
void RUDP_send(net_header_t* header, const void* payload, callback_function_t function, RUDP_transmission_result_t* result);
void RUDP_sendto(C_addr_t to, uint8_t priority, uint8_t app, uint8_t type, const void* payload, size_t payload_length, callback_function_t function, RUDP_transmission_result_t* result);
void RUDP_sendto2(C_addr_t to, uint8_t priority, uint8_t app, uint8_t type, const void* payload1, size_t payload_length1, const void* payload2, size_t payload_length2, callback_function_t function, RUDP_transmission_result_t* result);
size_t RUDP_max_payload_size();

#endif	// RUDP_LIGHT_H
