/**
 * @file service_management.h
 *
 * @brief Service management (header).
 *
 * Copyright (c) 2007, 2008, 2009, 2010 Georg Wittenburg, Freie Universitaet
 * Berlin (wittenbu@inf.fu-berlin.de). All rights reserved.
 * Please refer to the COPYING file for detailed licensing information.
 *
 * @author Georg Wittenburg
 *
 * $Id: service_management.h 3177 2010-01-05 16:43:45Z wittenbu $
 */

#ifndef SERVICE_MANAGEMENT_H
#define SERVICE_MANAGEMENT_H

#include "../types.h"
#include "service_types.h"

void alloc_service(SERVICE_service_t** service, const SERVICE_id_t* service_id, size_t service_data_size, float sync_traffic_fraction);
void start_service(SERVICE_service_t* service, uint8_t placement_algorithm_id);
void stop_service(SERVICE_service_t* service);
void free_service(SERVICE_service_t** service);
void free_service_list(void* p);
void update_service_data(SERVICE_service_t* service, uint16_t offset, size_t size, const char* data);
bool exists_service(const SERVICE_service_t* service);
bool is_running_service(const SERVICE_service_t* service);
bool is_redirecting_service(const SERVICE_service_t* service);
void lock_service(SERVICE_service_t* service);
void unlock_service(SERVICE_service_t* service);
bool is_locked_service(const SERVICE_service_t* service);
SERVICE_service_t* get_service(const SERVICE_id_t* service_id);
SERVICE_service_t* get_running_service(const SERVICE_id_t* service_id);

#endif	// SERVICE_MANAGEMENT_H
