/**
 * @file service_replication.h
 *
 * @brief Service replication (header).
 *
 * Copyright (c) 2007, 2008, 2009, 2010 Georg Wittenburg, Freie Universitaet
 * Berlin (wittenbu@inf.fu-berlin.de). All rights reserved.
 * Please refer to the COPYING file for detailed licensing information.
 *
 * @author Georg Wittenburg
 *
 * $Id: service_replication.h 3177 2010-01-05 16:43:45Z wittenbu $
 */

#ifndef SERVICE_REPLICATION_H
#define SERVICE_REPLICATION_H

#include "../types.h"
#include "service_types.h"

void abort_outgoing_replication(SERVICE_outgoing_replication_t* outgoing_replication, C_errno_t error_code);
void abort_incoming_replication(SERVICE_incoming_replication_t* incoming_replication);
void handle_SERVICE_REPLICATION_REQUEST(const net_header_t* header, const SERVICE_REPLICATION_REQUEST_packet_t* request, void* placement_algorithm_data, size_t placement_algorithm_data_size);
void handle_SERVICE_REPLICATION_DATA(const net_header_t* header, const SERVICE_REPLICATION_DATA_packet_t* data_header, const char* segment_data, size_t segment_size);
void handle_SERVICE_REPLICATION_ACK(const net_header_t* header, const SERVICE_REPLICATION_ACK_packet_t* ack);
void handle_SERVICE_REPLICATION_NACK(const net_header_t* header, const SERVICE_REPLICATION_NACK_packet_t* nack);
void replicate_service(SERVICE_service_t* service, C_addr_t destination_address, bool is_migration, callback_function_t callback, SERVICE_replication_result_t* result);
void migrate_service(SERVICE_service_t* service, C_addr_t destination_address, callback_function_t callback, SERVICE_migration_result_t* result);

#endif	// SERVICE_REPLICATION_H
