/**
 * @file list.h
 *
 * @brief List functions (header).
 *
 * Copyright (c) 2007, 2008, 2009, 2010 Georg Wittenburg, Freie Universitaet
 * Berlin (wittenbu@inf.fu-berlin.de). All rights reserved.
 * Please refer to the COPYING file for detailed licensing information.
 *
 * @author Georg Wittenburg
 *
 * $Id: list.h 3177 2010-01-05 16:43:45Z wittenbu $
 */

#ifndef UTIL_LIST_H
#define UTIL_LIST_H

#include "../types.h"

void list_alloc(list_t** list);
void list_free(list_t** list);
void list_free_including_data(list_t** list, void(*free_data)(void*));
void list_add(list_t* list, void* data);
void list_insert_where(list_t* list, void* data, bool(*matches)(const void*, const void*));
void list_append(list_t* dst_list, const list_t* src_list);
void list_merge(list_t* dst_list, const list_t* src_list);
void list_remove(list_t* list, const void* data);
void* list_remove_head(list_t* list);
void list_clear(list_t* list);
void list_clear_including_data(list_t* list, void(*free_data)(void*));
bool list_contains(const list_t* list, const void* data);
void* list_get(const list_t* list, unsigned int index);
unsigned int list_size(const list_t* list);
bool list_empty(const list_t* list);
void* list_head(const list_t* list);
void* list_tail(const list_t* list);
void* list_find(const list_t* list, const void* data, bool(*matches)(const void*, const void*));
void list_print(uint8_t debug_level, uint8_t msg_type, const list_t* list, void(*print_data)(uint8_t, uint8_t, const void*));
void list_map(const list_t* list, void(*function)(const void*));
void list_sort(list_t* list, bool(*greater_than)(const void*, const void*));
void list_iterator_init(const list_t* list, list_iterator_t* iterator);
void* list_iterator_next(list_iterator_t* iterator);
void list_to_array(const list_t* list, void* array, size_t data_item_size);
void list_from_array(list_t* list, const void* array, size_t data_item_size, size_t array_size);

#endif	// UTIL_LIST_H
