/**
 * @file net.h
 *
 * @brief Network utility functions (header).
 *
 * Copyright (c) 2007, 2008, 2009, 2010 Georg Wittenburg, Freie Universitaet
 * Berlin (wittenbu@inf.fu-berlin.de). All rights reserved.
 * Please refer to the COPYING file for detailed licensing information.
 *
 * @author Georg Wittenburg
 *
 * $Id: net.h 3368 2010-02-28 16:10:09Z gwitten $
 */

#ifndef UTIL_NET_H
#define UTIL_NET_H

#include "../types.h"

char* addr_to_str(C_addr_t address);
bool is_valid_address(C_addr_t address);
bool matches_address(const void* item, const void* cmp_item);
void alloc_header(net_header_t** header, C_addr_t destination_address, uint8_t priority, uint8_t app, uint8_t type, size_t payload_length);
void alloc_header_from_header(net_header_t** new_header, const net_header_t* orig_header);
void free_header(net_header_t** header);
void C_sendto(C_addr_t destination_address, uint8_t priority, uint8_t app, uint8_t type, const void* payload, size_t payload_length);
void C_sendto2(C_addr_t destination_address, uint8_t priority, uint8_t app, uint8_t type, const void* payload1, size_t payload_length1, const void* payload2, size_t payload_length2);
void forward(const net_header_t* header, const void* payload);
#ifdef C_USE_NETWORK_MAPPING
#ifdef C_ENABLE_BEACONING
float calculate_ETX(C_addr_t address);
void beacon_callback(callback_parameter_t parameter);
void handle_BEACON(const net_header_t* header, const void* payload);
#endif	// C_ENABLE_BEACONING
#endif	// C_USE_NETWORK_MAPPING
bool check_for_duplicate(packet_fingerprint_t* cache, uint16_t cache_size, const net_header_t* header);

void alloc_node(node_t** node, C_addr_t address);
void init_node(node_t* node, C_addr_t address);
void free_node(node_t** node);
void free_node_list(void* node);
bool matches_node_address(const void* item, const void* cmp_item);
bool node_address_less_than(const void* item, const void* cmp_item);
void print_node_address(uint8_t debug_level, uint8_t msg_type, const void* item);

node_t* register_known_node(C_addr_t address);
void register_known_link(C_addr_t src_address, C_addr_t dst_address);
void register_reportedly_known_link(C_addr_t src_address, C_addr_t dst_address);
void register_broken_link(C_addr_t src_address, C_addr_t dst_address);
void register_sent_packet(const net_header_t* header);
void register_received_packet(const net_header_t* header);
bool exists_known_node(C_addr_t address);
node_t* get_known_node(C_addr_t address);
uint16_t node_address_to_id(C_addr_t address);
C_addr_t node_id_to_address(uint16_t node_id);

void alloc_datarate_stat(datarate_stat_t** data);
void init_datarate_stat(datarate_stat_t* data);
void free_datarate_stat(datarate_stat_t** data);
void add_bytes_to_datarate_stat(datarate_stat_t* data, uint32_t bytes);
void set_additional_datarate_for_datarate_stats(datarate_stat_t* data, uint32_t datarate);
void add_datarate_stats(datarate_stat_t* dst_data, datarate_stat_t* src_data);
void subtract_datarate_stats(datarate_stat_t* dst_data, datarate_stat_t* src_data);
uint32_t calculate_datarate(datarate_stat_t* data);

#endif	// UTIL_NET_H
