#!/usr/bin/perl

##
#
# This script runs a given command on all machines in the network.
#
# Copyright (c) 2007, 2008, 2009, 2010 Georg Wittenburg, Freie Universitaet
# Berlin (wittenbu@inf.fu-berlin.de). All rights reserved.
# Please refer to the COPYING file for detailed licensing information.
#
# $Id: cmd.pl 3457 2010-04-21 07:26:49Z gwitten $
#
##

use strict;
use Fcntl ':flock';
use Term::ReadKey;

# List of machines is at https://www.mi.fu-berlin.de/w/IT/ServicesStudentPools.
my @HOSTS = (
	# Takustr. 9, K38 "China" (10 hosts)
	"shanghai.imp.fu-berlin.de",
	"peking.imp.fu-berlin.de",
	"guangzhou.imp.fu-berlin.de",
	"harbin.imp.fu-berlin.de",
	"chongqing.imp.fu-berlin.de",
	"shenyang.imp.fu-berlin.de",
	"wuhan.imp.fu-berlin.de",
	"chengdu.imp.fu-berlin.de",
	"tianjin.imp.fu-berlin.de",
	"xian.imp.fu-berlin.de",
	# Compute servers.
	"knecht.imp.fu-berlin.de",
	# Takustr. 9, K48 "Wales" (10 hosts)
	"llanberis.imp.fu-berlin.de",
	"corris.imp.fu-berlin.de",
	"bala.imp.fu-berlin.de",
	"tywyn.imp.fu-berlin.de",
	"porthmadog.imp.fu-berlin.de",
	"harlech.imp.fu-berlin.de",
	"ffestiniog.imp.fu-berlin.de",
	"welshpool.imp.fu-berlin.de",
	"aberystwyth.imp.fu-berlin.de",
	"llangollen.imp.fu-berlin.de",
	# Arnimallee 6, Raum 005 (7 hosts)
	"poncelet.imp.fu-berlin.de",
	"aiken.imp.fu-berlin.de",
	"mises.imp.fu-berlin.de",
	"kronecker.imp.fu-berlin.de",
	# "russel.imp.fu-berlin.de",	21.4.10: Unable to resolve host name.
	"banach.imp.fu-berlin.de",
	"bachet.imp.fu-berlin.de",
	# Arnimallee 6, Raum 030 (18 hosts)
	"rellich.imp.fu-berlin.de",
	"lalla.imp.fu-berlin.de",
	"cauchy.imp.fu-berlin.de",
	"breteuil.imp.fu-berlin.de",
	"hayes.imp.fu-berlin.de",
	"euler.imp.fu-berlin.de",
	"rudin.imp.fu-berlin.de",
	"germain.imp.fu-berlin.de",
	"fenchel.imp.fu-berlin.de",
	"hamel.imp.fu-berlin.de",
	"kendall.imp.fu-berlin.de",
	"neumann.imp.fu-berlin.de",
	"runge.imp.fu-berlin.de",
	"kutta.imp.fu-berlin.de",
	"peter.imp.fu-berlin.de",
	"landau.imp.fu-berlin.de",
	"hypathia.imp.fu-berlin.de",
	"jordan.imp.fu-berlin.de",
	# Takustr. 9, K60 "Niedersachsen" (9 hosts)
# 	"wolfsburg.imp.fu-berlin.de",
# 	"hannover.imp.fu-berlin.de",
# 	"salzgitter.imp.fu-berlin.de",
# 	"oldenburg.imp.fu-berlin.de",
# 	"braunschweig.imp.fu-berlin.de",
# 	"emden.imp.fu-berlin.de",
# 	"cuxhaven.imp.fu-berlin.de",
# 	"goslar.imp.fu-berlin.de",
# 	"wilhelmshaven.imp.fu-berlin.de",
	# DES testbed routers (according to uhu.mi.fu-berlin.de/etc/meshrouters)
# 	"t9-004a.des-mesh.net",
# 	"t9-004b.des-mesh.net",
# 	"t9-004c.des-mesh.net",
# 	"t9-006.des-mesh.net",
# 	"t9-007.des-mesh.net",
# 	"t9-009.des-mesh.net",
# 	"t9-011.des-mesh.net",
# 	"t9-018.des-mesh.net",
# 	"t9-020.des-mesh.net",
# 	"t9-022a.des-mesh.net",
# 	"t9-035.des-mesh.net",
# 	"t9-040.des-mesh.net",
# 	"t9-105.des-mesh.net",
# 	"t9-106.des-mesh.net",
# 	"t9-108.des-mesh.net",
# 	"t9-111.des-mesh.net",
# 	"t9-113.des-mesh.net",
# 	"t9-117.des-mesh.net",
# 	"t9-124.des-mesh.net",
# 	"t9-134.des-mesh.net",
# 	"t9-136.des-mesh.net",
# 	"t9-137.des-mesh.net",
# 	"t9-146.des-mesh.net",
# 	"t9-149.des-mesh.net",
# 	"t9-150.des-mesh.net",
# 	"t9-154.des-mesh.net",
# 	"t9-155.des-mesh.net",
# 	"t9-158.des-mesh.net",
# 	"t9-160.des-mesh.net",
# 	"t9-162.des-mesh.net",
# 	"t9-163.des-mesh.net",
# 	"t9-165.des-mesh.net",
# 	"t9-166.des-mesh.net",
# 	"t9-169.des-mesh.net",
# 	"t9-k21a.des-mesh.net",
# 	"t9-k21b.des-mesh.net",
# 	"t9-k23.des-mesh.net",
# 	"t9-k40.des-mesh.net",
# 	"t9-k60a.des-mesh.net",
# 	"t9-k60b.des-mesh.net",
# 	"t9-k61.des-mesh.net",
# 	"t9-k63.des-mesh.net",
# 	"t9-157t.des-mesh.net",
# 	"a6-008.des-mesh.net",
# 	"a6-009.des-mesh.net",
# 	"a6-015.des-mesh.net",
# 	"a6-017.des-mesh.net",
# 	"a6-031.des-mesh.net",
# 	"a6-032a.des-mesh.net",
# 	"a6-032b.des-mesh.net",
# 	"a6-102.des-mesh.net",
# 	"a6-107.des-mesh.net",
# 	"a6-108a.des-mesh.net",
# 	"a6-108b.des-mesh.net",
# 	"a6-124.des-mesh.net",
# 	"a6-126.des-mesh.net",
# 	"a6-139.des-mesh.net",
# 	"a6-207.des-mesh.net",
# 	"a6-212a.des-mesh.net",
# 	"a6-213.des-mesh.net",
# 	"a6-215.des-mesh.net",
);

my $USER = "gwitten";
# my $USER = "root";
# Only use public key auth, set connection timeout, and disable X11 forwarding.
my $SSH_OPTIONS = "-o PreferredAuthentications=publickey -o ConnectTimeout=10 -x";
my ($WCHAR, $HCHAR, $WPIXELS, $HPIXELS) = GetTerminalSize();
# my $WCHAR = 80;

if(scalar(@ARGV) < 1) {
	print "Usage:\tcmd.pl command [args]\n\n";
	exit(1);
}

my $cmd = join(" ", @ARGV);
for my $host (@HOSTS) {
	my $bar_length = $WCHAR - length($host) - 5;
	my $bar = "";
	for (1..$bar_length) {
		$bar .= "-";
	}
	my $separator = "$bar $host ---";
	print "$separator\n";
	print `ssh $SSH_OPTIONS $USER\@$host $cmd`;
}

my $complete_str = "[".(scalar(@HOSTS))." command(s) issued]";
my $bar_length = $WCHAR - length($complete_str) - 5;
my $bar = "";
for (1..$bar_length) {
	$bar .= "-";
}
my $footer = "$bar $complete_str ---";
print "$footer\n";
