#!/usr/bin/perl

##
#
# This script runs the service placement binary on several hosts with
# corresponding parameter files.
#
# Example:	eval_run.pl srvplace /home/fenn/gwitten/evaluation_commands/
#
# Copyright (c) 2007, 2008, 2009, 2010 Georg Wittenburg, Freie Universitaet
# Berlin (wittenbu@inf.fu-berlin.de). All rights reserved.
# Please refer to the COPYING file for detailed licensing information.
#
# $Id: eval_run.pl 3458 2010-04-21 09:00:45Z gwitten $
#
##

use strict;
use Cwd qw(abs_path chdir);
use Fcntl ':flock';

# List of machines is at https://www.mi.fu-berlin.de/w/IT/ServicesStudentPools.
my @HOSTS = (
	# Takustr. 9, K38 "China" (10 hosts)
	"shanghai.imp.fu-berlin.de",
	"peking.imp.fu-berlin.de",
	"guangzhou.imp.fu-berlin.de",
	"harbin.imp.fu-berlin.de",
	"chongqing.imp.fu-berlin.de",
	"shenyang.imp.fu-berlin.de",
	"wuhan.imp.fu-berlin.de",
	"chengdu.imp.fu-berlin.de",
	"tianjin.imp.fu-berlin.de",
	"xian.imp.fu-berlin.de",
	# Compute servers.
# 	"knecht.imp.fu-berlin.de",
	# Takustr. 9, K48 "Wales" (10 hosts)
	"llanberis.imp.fu-berlin.de",
	"corris.imp.fu-berlin.de",
	"bala.imp.fu-berlin.de",
	"tywyn.imp.fu-berlin.de",
	"porthmadog.imp.fu-berlin.de",
	"harlech.imp.fu-berlin.de",
	"ffestiniog.imp.fu-berlin.de",
	"welshpool.imp.fu-berlin.de",
	"aberystwyth.imp.fu-berlin.de",
	"llangollen.imp.fu-berlin.de",
	# Arnimallee 6, Raum 005 (7 hosts)
	"poncelet.imp.fu-berlin.de",
	"aiken.imp.fu-berlin.de",
	"mises.imp.fu-berlin.de",
	"kronecker.imp.fu-berlin.de",
	# "russel.imp.fu-berlin.de",	21.4.10: Unable to resolve host name.
	"banach.imp.fu-berlin.de",
	"bachet.imp.fu-berlin.de",
	# Arnimallee 6, Raum 030 (18 hosts)
	"rellich.imp.fu-berlin.de",
	"lalla.imp.fu-berlin.de",
	"cauchy.imp.fu-berlin.de",
	"breteuil.imp.fu-berlin.de",
	"hayes.imp.fu-berlin.de",
	"euler.imp.fu-berlin.de",
	"rudin.imp.fu-berlin.de",
	"germain.imp.fu-berlin.de",
	"fenchel.imp.fu-berlin.de",
	"hamel.imp.fu-berlin.de",
	"kendall.imp.fu-berlin.de",
	"neumann.imp.fu-berlin.de",
	"runge.imp.fu-berlin.de",
	"kutta.imp.fu-berlin.de",
	"peter.imp.fu-berlin.de",
	"landau.imp.fu-berlin.de",
	"hypathia.imp.fu-berlin.de",
	"jordan.imp.fu-berlin.de",
	# Takustr. 9, K60 "Niedersachsen" (9 hosts)
# 	"wolfsburg.imp.fu-berlin.de",
# 	"hannover.imp.fu-berlin.de",
# 	"salzgitter.imp.fu-berlin.de",
# 	"oldenburg.imp.fu-berlin.de",
# 	"braunschweig.imp.fu-berlin.de",
# 	"emden.imp.fu-berlin.de",
# 	"cuxhaven.imp.fu-berlin.de",
# 	"goslar.imp.fu-berlin.de",
# 	"wilhelmshaven.imp.fu-berlin.de",
	# DES testbed routers (according to uhu.mi.fu-berlin.de/etc/meshrouters)
# 	"t9-004a.des-mesh.net",
# 	"t9-004b.des-mesh.net",
# 	"t9-004c.des-mesh.net",
# 	"t9-006.des-mesh.net",
# 	"t9-007.des-mesh.net",
# 	"t9-009.des-mesh.net",
# 	"t9-011.des-mesh.net",
# 	"t9-018.des-mesh.net",
# 	"t9-020.des-mesh.net",
# 	"t9-022a.des-mesh.net",
# 	"t9-035.des-mesh.net",
# 	"t9-040.des-mesh.net",
# 	"t9-105.des-mesh.net",
# 	"t9-106.des-mesh.net",
# 	"t9-108.des-mesh.net",
# 	"t9-111.des-mesh.net",
# 	"t9-113.des-mesh.net",
# 	"t9-117.des-mesh.net",
# 	"t9-124.des-mesh.net",
# 	"t9-134.des-mesh.net",
# 	"t9-136.des-mesh.net",
# 	"t9-137.des-mesh.net",
# 	"t9-146.des-mesh.net",
# 	"t9-149.des-mesh.net",
# 	"t9-150.des-mesh.net",
# 	"t9-154.des-mesh.net",
# 	"t9-155.des-mesh.net",
# 	"t9-158.des-mesh.net",
# 	"t9-160.des-mesh.net",
# 	"t9-162.des-mesh.net",
# 	"t9-163.des-mesh.net",
# 	"t9-165.des-mesh.net",
# 	"t9-166.des-mesh.net",
# 	"t9-169.des-mesh.net",
# 	"t9-k21a.des-mesh.net",
# 	"t9-k21b.des-mesh.net",
# 	"t9-k23.des-mesh.net",
# 	"t9-k40.des-mesh.net",
# 	"t9-k60a.des-mesh.net",
# 	"t9-k60b.des-mesh.net",
# 	"t9-k61.des-mesh.net",
# 	"t9-k63.des-mesh.net",
# 	"t9-157t.des-mesh.net",
# 	"a6-008.des-mesh.net",
# 	"a6-009.des-mesh.net",
# 	"a6-015.des-mesh.net",
# 	"a6-017.des-mesh.net",
# 	"a6-031.des-mesh.net",
# 	"a6-032a.des-mesh.net",
# 	"a6-032b.des-mesh.net",
# 	"a6-102.des-mesh.net",
# 	"a6-107.des-mesh.net",
# 	"a6-108a.des-mesh.net",
# 	"a6-108b.des-mesh.net",
# 	"a6-124.des-mesh.net",
# 	"a6-126.des-mesh.net",
# 	"a6-139.des-mesh.net",
# 	"a6-207.des-mesh.net",
# 	"a6-212a.des-mesh.net",
# 	"a6-213.des-mesh.net",
# 	"a6-215.des-mesh.net",
);

my $USER = "gwitten";
# my $USER = "root";
# Only use public key auth, set connection timeout, and disable X11 forwarding.
my $SSH_OPTIONS = "-o PreferredAuthentications=publickey -o ConnectTimeout=10 -x";
my $SYNC_FILE_NAME = "eval_run.tmp";
my $OUTPUT_RUNNING_SUFFIX = "_out.txt_RUNNING";
my $OUTPUT_DONE_SUFFIX = "_out.txt";

sub print_time {
	my($string) = @_;
	my($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime(time);
	$sec = ($sec < 10) ? "0$sec" : $sec;
	$min = ($min < 10) ? "0$min" : $min;
	$hour = ($hour < 10) ? "0$hour" : $hour;
	$mday = ($mday < 10) ? "0$mday" : $mday;
	$mon++;
	$mon = ($mon < 10) ? "0$mon" : $mon;
	$year += 1900;
	print "$mday/$mon $hour:$min:$sec\t$string";
}

sub check_host {
	my($host, $check_if_busy, $print_status) = @_;

	`ping -c1 -q $host`;
	if($? != 0) {
		if($print_status) {print "NOT REACHABLE";}
		return 0;
	}

	my $ps_wc_str = `ssh $SSH_OPTIONS $USER\@$host "/bin/ps aux | grep $USER | grep ns | grep -v '>>' | grep -v grep | wc"`;
	if(($? != 256) && ($? != 0)) {
		if($print_status) {print "LOGIN FAILURE";}
		return 0;
	}

	if($check_if_busy) {
		chomp $ps_wc_str;
		$ps_wc_str =~ /(\d+).+\d+.+\d+/o;
		my $ns_count = $1;
		if($ns_count > 0) {
			if($print_status) {print "RUNNING ($ns_count)";}
			return 0;
		}
	}

	my $uptime_str = `ssh $SSH_OPTIONS $USER\@$host uptime`;
	if($? != 0) {
		if($print_status) {print "LOGIN FAILURE";}
		return 0;
	}

	if($check_if_busy) {
		chomp $uptime_str;
		$uptime_str =~ /([\.0-9]*), ([\.0-9]*), ([\.0-9]*)$/o;
		my $load_average_1m = $1;
		my $load_average_5m = $2;
		my $load_average_15m = $3;
		if(
			($load_average_1m > 2.5)
			&& ($load_average_5m > 2.5)
			&& ($load_average_15m > 2.5)
		) {
			if($print_status) {print "BUSY ($load_average_1m, $load_average_5m, $load_average_15m)";}
			return 0;
		}
	}

	if($print_status) {print "OK";}
	return 1;
}

if(scalar(@ARGV) < 3) {
	print "Usage:\teval_run.pl srvplace_executable_with_path\n";
	print "\t\tabsolute_path_to_remote_commands_directory\n";
	print "\t\tabsolute_path_to_remote_output_directory\n";
	print "\t\t[absolute_path_to_remote_topology_directory]\n\n";
	exit(1);
}

my $srvplace_executable = $ARGV[0];
my $commands_directory = $ARGV[1];
if($commands_directory !~ /\/\Z/o) {
	$commands_directory .= '/';
}
if($commands_directory =~ / /o) {
	print "ERROR: commands_directory may not contain whitespaces.\n";
	exit(2);
}
my $output_directory = $ARGV[2];
if($output_directory !~ /\/\Z/o) {
	$output_directory .= '/';
}
if($output_directory =~ / /o) {
	print "ERROR: output_directory may not contain whitespaces.\n";
	exit(2);
}
my $topology_directory = "";
if(scalar(@ARGV) == 4) {
	$topology_directory = $ARGV[3];
	if($topology_directory !~ /\/\Z/o) {
		$topology_directory .= '/';
	}
	if($topology_directory =~ / /o) {
		print "ERROR: topology_directory may not contain whitespaces.\n";
		exit(2);
	}
}

print "\n";
print_time "Starting evaluation using $srvplace_executable.\n";
print_time "Command files are in $commands_directory.\n";
print_time "Output will be generated in $output_directory.\n";
if($topology_directory ne "") {
	print_time "Topology files are in $topology_directory.\n";
}
print "\n";

if(-e $SYNC_FILE_NAME) {
	print_time "Removing stale sync file ($SYNC_FILE_NAME)...\n";
	unlink $SYNC_FILE_NAME;
	print "\n";
}

opendir(my $DIR, ".") or die $!;
while(my $file = readdir($DIR)) {
	if($file =~ /$OUTPUT_RUNNING_SUFFIX\Z/) {
		print_time "Removing stale output file ($file)...\n";
		unlink $file;
	}
}
closedir($DIR);

print_time "Scanning for evaluation commands...\n";
my @command_files;
opendir(my $DIR, ".") or die $!;
while(my $file = readdir($DIR)) {
	if($file =~ /(.+)_cmnds\.txt\Z/) {
		if(!-e $output_directory."/".$1.$OUTPUT_DONE_SUFFIX) {
			push(@command_files, $file);
		} else {
			print_time "Output for command file $file already exists, aborting...\n";
			print "\n";
			exit();
		}
	}
}
closedir($DIR);
@command_files = sort(@command_files);
if(scalar(@command_files) > 0) {
	print_time "Found ".scalar(@command_files)." command file(s).\n";
	print "\n";
} else {
	my $pwd = `pwd`;
	chomp($pwd);
	print_time "No command files found in $pwd, aborting...\n";
	print "\n";
	exit();
}

print_time "Checking which hosts are up:\n";
my @potential_hosts = @HOSTS;
if($topology_directory ne "") {
	@potential_hosts = ();
	open(HOSTS, "$topology_directory/hosts.txt") or die "Can't open file \"$topology_directory/hosts.txt\" for reading";
	for my $host (<HOSTS>) {
		chomp($host);
		push(@potential_hosts, $host);
	}	
	close(HOSTS);
}
my @hosts;
for my $host (@potential_hosts) {
	print_time "Checking $host... ";
	if(check_host($host, 0, 1)) {
		print ", adding to list...\n";
		push(@hosts, $host);
	} else {
		print ", ignoring...\n";
	}
}
if(scalar(@hosts) == scalar(@command_files)) {
	print_time "".scalar(@hosts)." host(s) are available.\n";
	print "\n";
} elsif(scalar(@hosts) > scalar(@command_files)) {
	print_time "".scalar(@hosts)." host(s) are available.\n";
	print "\n";
	print_time "WARNING: ".(scalar(@hosts) - scalar(@command_files))." host(s) are unused.\n";
	print "\n";
	while(scalar(@hosts) > scalar(@command_files)) {
		pop(@hosts);
	}
} else {
	print_time "".scalar(@hosts)." hosts are available, but ".scalar(@command_files)." are required. Aborting...\n";
	print "\n";
	exit();
}

open(my $SYNC_FILE, "> $SYNC_FILE_NAME") or die $!;
print $SYNC_FILE "0";
close($SYNC_FILE);

my $process_count = 0;
my $pid;
while($process_count < scalar(@hosts)) {
	$process_count++;
	$pid = fork();
	if($pid == 0) {
		last;
	}
}

if($pid != 0) {
	#
	# Parent process.
	#
	while($process_count) {
		wait();
		$process_count--;
		print_time "$process_count host(s) still running...\n";
	}
	unlink $SYNC_FILE_NAME;
	print_time "All evaluation hosts are done.\n";
	exit();
}

#
# Child process.
#

# Retrieve an unique id.
sleep 1;	# Avoid race conditions with parent.
my $id;
open(my $SYNC_FILE, "+< $SYNC_FILE_NAME") or die $!;
flock($SYNC_FILE, LOCK_EX) or die $!;
sleep 1;	# Avoid race conditions later on.
$id = <$SYNC_FILE>;
if(!defined $id) {die "Didn't find an ID in sync file";}
seek($SYNC_FILE, 0, 0);
print $SYNC_FILE ($id + 1);
flock($SYNC_FILE, LOCK_UN);
close($SYNC_FILE);

# Wait some time to start all nodes simultaneously.
sleep (scalar(@hosts) - $id);

# Set local host.
my $host = $hosts[$id];
my $command_file = $command_files[$id];
$command_file =~ /(.+)_cmnds\.txt\Z/o;
my $command_file_name = $1;

my $output_running = $command_file_name.$OUTPUT_RUNNING_SUFFIX;
my $output_done = $command_file_name.$OUTPUT_DONE_SUFFIX;

my $progess_info = "".($id + 1)."/".scalar(@command_files);
if((!-e $output_running) && (!-e $output_done)) {
	print_time "$host: $command_file_name -> START ($progess_info)\n";

	print `ssh $SSH_OPTIONS $USER\@$host "echo $host: $srvplace_executable $command_file > $output_directory/$output_running"`;
	# NOTE: This should be "&>>" instead of ">>" to
	# redirect and append. But bash doesn't support this at
	# the moment.
	print `ssh $SSH_OPTIONS $USER\@$host "cd $commands_directory; $srvplace_executable --file $command_file >> $output_directory/$output_running"`;
	print_time "$host: $command_file_name -> STOP ($progess_info)\n";
	if($? == 0) {
		print `ssh $SSH_OPTIONS $USER\@$host "mv $output_directory/$output_running $output_directory/$output_done"`;
	} else {
		my($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime(time);
		$sec = ($sec < 10) ? "0$sec" : $sec;
		$min = ($min < 10) ? "0$min" : $min;
		$hour = ($hour < 10) ? "0$hour" : $hour;
		$mday = ($mday < 10) ? "0$mday" : $mday;
		$mon++;
		$mon = ($mon < 10) ? "0$mon" : $mon;
		$year += 1900;
		print `ssh $SSH_OPTIONS $USER\@$host "mv $output_directory/$output_running $output_directory/$output_done\_ERROR_$?\_$host\_$year$mon$mday$hour$min$sec"`;
		print_time "===> $host: $command_file_name -> ERROR ($progess_info terminated with $?) <===\n";
	}
}

exit();
