#!/usr/bin/perl

use strict;

# DES testbed routers (according to uhu.mi.fu-berlin.de/etc/meshrouters)
my @HOSTS = (
	"t9-004a.des-mesh.net",
	"t9-004b.des-mesh.net",
	"t9-004c.des-mesh.net",
	"t9-006.des-mesh.net",
	"t9-007.des-mesh.net",
	"t9-009.des-mesh.net",
	"t9-011.des-mesh.net",
	"t9-018.des-mesh.net",
	"t9-020.des-mesh.net",
	"t9-022a.des-mesh.net",
	"t9-035.des-mesh.net",
	"t9-040.des-mesh.net",
	"t9-105.des-mesh.net",
	"t9-106.des-mesh.net",
	"t9-108.des-mesh.net",
	"t9-111.des-mesh.net",
	"t9-113.des-mesh.net",
	"t9-117.des-mesh.net",
	"t9-124.des-mesh.net",
	"t9-134.des-mesh.net",
	"t9-136.des-mesh.net",
	"t9-137.des-mesh.net",
	"t9-146.des-mesh.net",
	"t9-149.des-mesh.net",
	"t9-150.des-mesh.net",
	"t9-154.des-mesh.net",
	"t9-155.des-mesh.net",
	"t9-158.des-mesh.net",
	"t9-160.des-mesh.net",
	"t9-162.des-mesh.net",
	"t9-163.des-mesh.net",
	"t9-165.des-mesh.net",
	"t9-166.des-mesh.net",
	"t9-169.des-mesh.net",
	"t9-k21a.des-mesh.net",
	"t9-k21b.des-mesh.net",
	"t9-k23.des-mesh.net",
	"t9-k40.des-mesh.net",
	"t9-k60a.des-mesh.net",
	"t9-k60b.des-mesh.net",
	"t9-k61.des-mesh.net",
	"t9-k63.des-mesh.net",
	"t9-157t.des-mesh.net",
# 	"a6-008.des-mesh.net",
# 	"a6-009.des-mesh.net",
# 	"a6-015.des-mesh.net",
# 	"a6-017.des-mesh.net",
# 	"a6-031.des-mesh.net",
# 	"a6-032a.des-mesh.net",
# 	"a6-032b.des-mesh.net",
# 	"a6-102.des-mesh.net",
# 	"a6-107.des-mesh.net",
# 	"a6-108a.des-mesh.net",
# 	"a6-108b.des-mesh.net",
# 	"a6-124.des-mesh.net",
# 	"a6-126.des-mesh.net",
# 	"a6-139.des-mesh.net",
# 	"a6-207.des-mesh.net",
# 	"a6-212a.des-mesh.net",
# 	"a6-213.des-mesh.net",
# 	"a6-215.des-mesh.net",
);

my $SRVPLACE = "/root/srvplace/srvplace.sp_none_TESTBED";
# my $SRVPLACE = "/root/srvplace/srvplace.sp_spi_TESTBED";
my $BASE_DIR = "/home/wittenbu/evaluations/placement_evaluation_none_TESTBED";
my $RUN_DIR = "/root/srvplace";
my $OUTPUT_DIR = "/tmp/srvplace";

my @SERVICE_SIZES = (10000);
my @SYNC_RATIOS = ("0.010");
# my @NODES = (16, 25, 36, 49, 64, 81, 100);
my @NODES = (36);
my @ACTIVE_NODE_RATIOS = (0.5);
my @REQUEST_FREQUENCIES = ("0.1", "0.5", "1.0", "2.0", "5.0");
my $RUNS = 30;

if(!-e $BASE_DIR) {
	print "Base directory $BASE_DIR does not exist.\n\n";
	exit(1);
}

for my $run (1..$RUNS) {
	if($run < 10) {
		$run = '0'.$run;
	}
for my $request_frequency (@REQUEST_FREQUENCIES) {
for my $active_node_ratio (@ACTIVE_NODE_RATIOS) {
for my $nodes (@NODES) {
for my $sync_ratio (@SYNC_RATIOS) {
for my $service_size (@SERVICE_SIZES) {

	chdir($BASE_DIR);

	print "Cleaning up output directories...\n";
	for my $host (@HOSTS) {
		print `ssh root\@$host rm -f $OUTPUT_DIR/*`;
	}

	my $dir = "service_size_$service_size/sync_ratio_$sync_ratio/nodes_$nodes/activity_factor_$active_node_ratio/request_frequency_$request_frequency/run_$run/";
	if(!-e $dir) {
		print "Run directory $dir does not exist.\n";
		exit(2);
	}
	chdir($dir);

	`ls *_out.txt >& /dev/null`;
	if($? == 0) {
		print "Output files already exist in $dir, skipping...\n";
		next;
	}

	print `scp *cmnds.txt root\@t9-157t.des-mesh.net:/root/srvplace/`;
	if($? != 0) {
		print "Error while copying command files from $dir.\n";
		exit(2);
	}

	my $start_time = time();
	print "\n[".localtime($start_time)."] Starting run for $dir...\n";
	print `eval_run.pl $SRVPLACE $RUN_DIR $OUTPUT_DIR`;
	my $run_errno = $?;
	my $stop_time = time();
	my $duration = $stop_time - $start_time;
	my $hours = int($duration / 3600);
	my $minutes = int($duration / 60);
	if($minutes < 10) {
		$minutes = '0'.$minutes;
	}
	my $seconds = $duration % 60;
	if($seconds < 10) {
		$seconds = '0'.$seconds;
	}
	if($run_errno != 0) {
		print "[".localtime($stop_time)."] Run for $dir failed after $hours:$minutes:$seconds ($run_errno).\n";
		print "Aborting...\n\n";
		exit(1);
	}
	print "[".localtime($stop_time)."] Run for $dir completed after $hours:$minutes:$seconds.\n";

	print "Collecting results...\n";
	for my $host (@HOSTS) {
		print `scp root\@$host:$OUTPUT_DIR/* .`;
	}

}
}
}
}
}
}
